#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest.mock import call, patch

import pytest

from airflow.exceptions import AirflowException
from airflow.providers.teradata.operators.teradata_compute_cluster import (
    TeradataComputeClusterDecommissionOperator,
    TeradataComputeClusterProvisionOperator,
    TeradataComputeClusterResumeOperator,
    TeradataComputeClusterSuspendOperator,
    _single_result_row_handler,
)
from airflow.providers.teradata.triggers.teradata_compute_cluster import TeradataComputeClusterSyncTrigger
from airflow.providers.teradata.utils.constants import Constants


@pytest.fixture
def compute_profile_name():
    return "test_profile"


@pytest.fixture
def compute_group_name():
    return "test_group"


@pytest.fixture
def query_strategy():
    return "test_query_strategy"


@pytest.fixture
def compute_map():
    return "test_compute_map"


@pytest.fixture
def compute_attribute():
    return "test_compute_attribute"


@pytest.fixture
def compute_cluster_provision_instance(compute_profile_name):
    return TeradataComputeClusterProvisionOperator(
        task_id="test", compute_profile_name=compute_profile_name, teradata_conn_id="test_conn"
    )


@pytest.fixture
def compute_cluster_decommission_instance(compute_profile_name):
    return TeradataComputeClusterDecommissionOperator(
        task_id="test", compute_profile_name=compute_profile_name, teradata_conn_id="test_conn"
    )


@pytest.fixture
def compute_cluster_resume_instance(compute_profile_name):
    return TeradataComputeClusterResumeOperator(
        task_id="test", compute_profile_name=compute_profile_name, teradata_conn_id="test_conn"
    )


@pytest.fixture
def compute_cluster_suspend_instance(compute_profile_name):
    return TeradataComputeClusterSuspendOperator(
        task_id="test", compute_profile_name=compute_profile_name, teradata_conn_id="test_conn"
    )


class TestTeradataComputeClusterOperator:
    def test_compute_cluster_execute_invalid_profile(self, compute_cluster_provision_instance):
        compute_cluster_provision_instance.compute_profile_name = None
        with pytest.raises(AirflowException):
            compute_cluster_provision_instance._compute_cluster_execute()

    def test_compute_cluster_execute_empty_profile(self, compute_cluster_provision_instance):
        compute_cluster_provision_instance.compute_profile_name = ""
        with pytest.raises(AirflowException):
            compute_cluster_provision_instance._compute_cluster_execute()

    def test_compute_cluster_execute_none_profile(self, compute_cluster_provision_instance):
        compute_cluster_provision_instance.compute_profile_name = "None"
        with pytest.raises(AirflowException):
            compute_cluster_provision_instance._compute_cluster_execute()

    def test_compute_cluster_execute_not_lake(self, compute_cluster_provision_instance):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = [None]
            with pytest.raises(AirflowException):
                compute_cluster_provision_instance._compute_cluster_execute()

    def test_compute_cluster_execute_not_lake_version_check(self, compute_cluster_provision_instance):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "19"]
            with pytest.raises(AirflowException):
                compute_cluster_provision_instance._compute_cluster_execute()

    def test_compute_cluster_execute_not_lake_version_none(self, compute_cluster_provision_instance):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", None]
            with pytest.raises(AirflowException):
                compute_cluster_provision_instance._compute_cluster_execute()

    def test_compute_cluster_execute_not_lake_version_invalid(self, compute_cluster_provision_instance):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "invalid"]
            with pytest.raises(AirflowException):
                compute_cluster_provision_instance._compute_cluster_execute()

    def test_compute_cluster_execute_complete_success(self, compute_cluster_provision_instance):
        event = {"status": "success", "message": "Success message"}
        # Call the method under test
        result = compute_cluster_provision_instance._compute_cluster_execute_complete(event)
        assert result == "Success message"

    def test_compute_cluster_execute_complete_error(self, compute_cluster_provision_instance):
        event = {"status": "error", "message": "Error message"}
        with pytest.raises(AirflowException):
            compute_cluster_provision_instance._compute_cluster_execute_complete(event)

    def test_cc_execute_provision_new_cp(self, compute_cluster_provision_instance):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", None, "Success"]
            compute_profile_name = compute_cluster_provision_instance.compute_profile_name
            with patch.object(compute_cluster_provision_instance, "defer") as mock_defer:
                # Assert that defer method is called with the correct parameters
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_provision_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    operation_type=Constants.CC_CREATE_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_provision_instance._compute_cluster_execute()
                assert result == "Success"
                mock_defer.assert_called_once()
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"CREATE COMPUTE PROFILE {compute_profile_name}",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_cc_execute_provision_exists_cp(self, compute_cluster_provision_instance):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "RUNNING", "Success"]
            compute_profile_name = compute_cluster_provision_instance.compute_profile_name
            with patch.object(compute_cluster_provision_instance, "defer") as mock_defer:
                # Assert that defer method is called with the correct parameters
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_provision_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    operation_type=Constants.CC_CREATE_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_provision_instance._compute_cluster_execute()
                assert result == "RUNNING"
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_cc_execute_provision_new_cp_exists_cg(
        self, compute_group_name, compute_cluster_provision_instance
    ):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "1", None, "Success"]
            compute_cluster_provision_instance.compute_group_name = compute_group_name
            compute_profile_name = compute_cluster_provision_instance.compute_profile_name
            with patch.object(compute_cluster_provision_instance, "defer") as mock_defer:
                # Assert that defer method is called with the correct parameters
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_provision_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    compute_group_name=compute_group_name,
                    operation_type=Constants.CC_CREATE_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_provision_instance._compute_cluster_execute()
                assert result == "Success"
                mock_defer.assert_called_once()
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SELECT  count(1) FROM DBC.ComputeGroups WHERE UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}') AND UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"CREATE COMPUTE PROFILE {compute_profile_name} IN {compute_group_name}",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_cc_execute_provision_exists_cp_exists_cg(
        self, compute_group_name, compute_cluster_provision_instance
    ):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "1", "RUNNING", "Success"]
            compute_profile_name = compute_cluster_provision_instance.compute_profile_name
            compute_cluster_provision_instance.compute_group_name = compute_group_name
            with patch.object(compute_cluster_provision_instance, "defer") as mock_defer:
                # Assert that defer method is called with the correct parameters
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_provision_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    compute_group_name=compute_group_name,
                    operation_type=Constants.CC_CREATE_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_provision_instance._compute_cluster_execute()
                assert result == "RUNNING"
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SELECT  count(1) FROM DBC.ComputeGroups WHERE UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}') AND UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_cc_execute_provision_new_cp_new_cg(self, compute_group_name, compute_cluster_provision_instance):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "0", "Success", None, "Success"]
            compute_cluster_provision_instance.compute_group_name = compute_group_name
            compute_profile_name = compute_cluster_provision_instance.compute_profile_name
            with patch.object(compute_cluster_provision_instance, "defer") as mock_defer:
                # Assert that defer method is called with the correct parameters
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_provision_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    compute_group_name=compute_group_name,
                    operation_type=Constants.CC_CREATE_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_provision_instance._compute_cluster_execute()
                assert result == "Success"
                mock_defer.assert_called_once()
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SELECT  count(1) FROM DBC.ComputeGroups WHERE UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"CREATE COMPUTE GROUP {compute_group_name}", handler=_single_result_row_handler
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}') AND UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"CREATE COMPUTE PROFILE {compute_profile_name} IN {compute_group_name}",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_cc_execute_provision_new_cp_new_cg_with_options(
        self,
        compute_group_name,
        query_strategy,
        compute_map,
        compute_attribute,
        compute_cluster_provision_instance,
    ):
        with patch.object(compute_cluster_provision_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "0", "Success", None, "Success"]
            compute_cluster_provision_instance.compute_group_name = compute_group_name
            compute_profile_name = compute_cluster_provision_instance.compute_profile_name
            compute_cluster_provision_instance.query_strategy = query_strategy
            compute_cluster_provision_instance.compute_map = compute_map
            compute_cluster_provision_instance.compute_attribute = compute_attribute

            with patch.object(compute_cluster_provision_instance, "defer") as mock_defer:
                # Assert that defer method is called with the correct parameters
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_provision_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    compute_group_name=compute_group_name,
                    operation_type=Constants.CC_CREATE_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_provision_instance._compute_cluster_execute()
                assert result == "Success"
                mock_defer.assert_called_once()
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SELECT  count(1) FROM DBC.ComputeGroups WHERE UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"CREATE COMPUTE GROUP {compute_group_name} USING QUERY_STRATEGY ('{query_strategy}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}') AND UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"CREATE COMPUTE PROFILE {compute_profile_name} IN {compute_group_name}, "
                            f"INSTANCE = {compute_map}, INSTANCE TYPE = {query_strategy} USING {compute_attribute}",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_compute_cluster_execute_drop_cp(self, compute_cluster_decommission_instance):
        with patch.object(compute_cluster_decommission_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", None]
            compute_profile_name = compute_cluster_decommission_instance.compute_profile_name
            compute_cluster_decommission_instance._compute_cluster_execute()
            mock_hook.run.assert_has_calls(
                [
                    call(
                        "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                        handler=_single_result_row_handler,
                    ),
                    call(f"DROP COMPUTE PROFILE {compute_profile_name}", handler=_single_result_row_handler),
                ]
            )

    def test_compute_cluster_execute_drop_cp_cg(
        self, compute_cluster_decommission_instance, compute_group_name
    ):
        with patch.object(compute_cluster_decommission_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", None, None]
            compute_profile_name = compute_cluster_decommission_instance.compute_profile_name
            compute_cluster_decommission_instance.compute_group_name = compute_group_name
            compute_cluster_decommission_instance.delete_compute_group = True
            compute_cluster_decommission_instance._compute_cluster_execute()
            mock_hook.run.assert_has_calls(
                [
                    call(
                        "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        f"DROP COMPUTE PROFILE {compute_profile_name} IN COMPUTE GROUP {compute_group_name}",
                        handler=_single_result_row_handler,
                    ),
                    call(f"DROP COMPUTE GROUP {compute_group_name}", handler=_single_result_row_handler),
                ]
            )

    def test_compute_cluster_execute_resume_success(self, compute_cluster_resume_instance):
        with patch.object(compute_cluster_resume_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "Suspended", "Success"]
            compute_profile_name = compute_cluster_resume_instance.compute_profile_name

            with patch.object(compute_cluster_resume_instance, "defer") as mock_defer:
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_resume_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    operation_type=Constants.CC_RESUME_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_resume_instance._compute_cluster_execute()
                assert result == "Success"
                mock_defer.assert_called_once()
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"RESUME COMPUTE FOR COMPUTE PROFILE {compute_profile_name}",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_compute_cluster_execute_resume_cg_success(
        self, compute_group_name, compute_cluster_resume_instance
    ):
        with patch.object(compute_cluster_resume_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "Suspended", "Success"]
            compute_profile_name = compute_cluster_resume_instance.compute_profile_name
            compute_cluster_resume_instance.compute_group_name = compute_group_name
            with patch.object(compute_cluster_resume_instance, "defer") as mock_defer:
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_resume_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    operation_type=Constants.CC_RESUME_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_resume_instance._compute_cluster_execute()
                assert result == "Success"
                mock_defer.assert_called_once()
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')"
                            f" AND UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"RESUME COMPUTE FOR COMPUTE PROFILE {compute_profile_name} IN COMPUTE GROUP {compute_group_name}",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_compute_cluster_execute_resume_cc_not_exists(
        self, compute_group_name, compute_cluster_resume_instance
    ):
        with patch.object(compute_cluster_resume_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", None]
            compute_profile_name = compute_cluster_resume_instance.compute_profile_name
            with pytest.raises(AirflowException):
                compute_cluster_resume_instance._compute_cluster_execute()
            mock_hook.run.assert_has_calls(
                [
                    call(
                        "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')",
                        handler=_single_result_row_handler,
                    ),
                ]
            )

    def test_compute_cluster_execute_resume_same_state(self, compute_cluster_resume_instance):
        with patch.object(compute_cluster_resume_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "Running"]
            compute_profile_name = compute_cluster_resume_instance.compute_profile_name
            result = compute_cluster_resume_instance._compute_cluster_execute()
            assert result is None
            mock_hook.run.assert_has_calls(
                [
                    call(
                        "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')",
                        handler=_single_result_row_handler,
                    ),
                ]
            )

    def test_compute_cluster_execute_suspend_success(self, compute_cluster_suspend_instance):
        with patch.object(compute_cluster_suspend_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "Running", "Success"]
            compute_profile_name = compute_cluster_suspend_instance.compute_profile_name

            with patch.object(compute_cluster_suspend_instance, "defer") as mock_defer:
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_suspend_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    operation_type=Constants.CC_RESUME_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_suspend_instance._compute_cluster_execute()
                assert result == "Success"
                mock_defer.assert_called_once()
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SUSPEND COMPUTE FOR COMPUTE PROFILE {compute_profile_name}",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_compute_cluster_execute_suspend_cg_success(
        self, compute_group_name, compute_cluster_suspend_instance
    ):
        with patch.object(compute_cluster_suspend_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "Running", "Success"]
            compute_profile_name = compute_cluster_suspend_instance.compute_profile_name
            compute_cluster_suspend_instance.compute_group_name = compute_group_name
            with patch.object(compute_cluster_suspend_instance, "defer") as mock_defer:
                expected_trigger = TeradataComputeClusterSyncTrigger(
                    teradata_conn_id=compute_cluster_suspend_instance.teradata_conn_id,
                    compute_profile_name=compute_profile_name,
                    operation_type=Constants.CC_RESUME_OPR,
                    poll_interval=Constants.CC_POLL_INTERVAL,
                )
                mock_defer.return_value = expected_trigger
                result = compute_cluster_suspend_instance._compute_cluster_execute()
                assert result == "Success"
                mock_defer.assert_called_once()
                mock_hook.run.assert_has_calls(
                    [
                        call(
                            "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')"
                            f" AND UPPER(ComputeGroupName) = UPPER('{compute_group_name}')",
                            handler=_single_result_row_handler,
                        ),
                        call(
                            f"SUSPEND COMPUTE FOR COMPUTE PROFILE {compute_profile_name} IN COMPUTE GROUP {compute_group_name}",
                            handler=_single_result_row_handler,
                        ),
                    ]
                )

    def test_compute_cluster_execute_suspend_cc_not_exists(
        self, compute_group_name, compute_cluster_suspend_instance
    ):
        with patch.object(compute_cluster_suspend_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", None]
            compute_profile_name = compute_cluster_suspend_instance.compute_profile_name
            with pytest.raises(AirflowException):
                compute_cluster_suspend_instance._compute_cluster_execute()
            mock_hook.run.assert_has_calls(
                [
                    call(
                        "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')",
                        handler=_single_result_row_handler,
                    ),
                ]
            )

    def test_compute_cluster_execute_suspend_same_state(self, compute_cluster_suspend_instance):
        with patch.object(compute_cluster_suspend_instance, "hook") as mock_hook:
            # Set up mock return values
            mock_hook.run.side_effect = ["1", "20.00", "Suspended"]
            compute_profile_name = compute_cluster_suspend_instance.compute_profile_name
            result = compute_cluster_suspend_instance._compute_cluster_execute()
            assert result is None
            mock_hook.run.assert_has_calls(
                [
                    call(
                        "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'",
                        handler=_single_result_row_handler,
                    ),
                    call(
                        f"SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('{compute_profile_name}')",
                        handler=_single_result_row_handler,
                    ),
                ]
            )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
from datetime import datetime
from unittest import mock

from airflow.models import Connection
from airflow.providers.teradata.hooks.teradata import TeradataHook, _handle_user_query_band_text


class TestTeradataHook:
    def setup_method(self):
        self.connection = Connection(
            conn_id="teradata_conn_id",
            conn_type="teradata",
            login="login",
            password="password",
            host="host",
            schema="schema",
        )
        self.db_hook = TeradataHook(teradata_conn_id="teradata_conn_id", database="test_db")
        self.db_hook.get_connection = mock.Mock()
        self.db_hook.get_connection.return_value = self.connection
        self.cur = mock.MagicMock(rowcount=0)
        self.conn = mock.MagicMock()
        self.conn.cursor.return_value = self.cur
        self.conn.extra_dejson = {}
        conn = self.conn

        class UnitTestTeradataHook(TeradataHook):
            def get_conn(self):
                return conn

            @classmethod
            def get_connection(cls, conn_id: str) -> Connection:
                return conn

        self.test_db_hook = UnitTestTeradataHook(teradata_conn_id="teradata_conn_id")

    @mock.patch("teradatasql.connect")
    def test_get_conn(self, mock_connect):
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"

    @mock.patch("teradatasql.connect")
    def test_get_tmode_conn(self, mock_connect):
        tmode_name = {"tmode": "tera"}
        self.connection.extra = json.dumps(tmode_name)
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"
        assert kwargs["tmode"] == "tera"

    @mock.patch("teradatasql.connect")
    def test_get_sslmode_conn(self, mock_connect):
        tmode_name = {"sslmode": "require"}
        self.connection.extra = json.dumps(tmode_name)
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"
        assert kwargs["sslmode"] == "require"

    @mock.patch("teradatasql.connect")
    def test_get_sslverifyca_conn(self, mock_connect):
        extravalues = {"sslmode": "verify-ca", "sslca": "/tmp/cert"}
        self.connection.extra = json.dumps(extravalues)
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"
        assert kwargs["sslmode"] == "verify-ca"
        assert kwargs["sslca"] == "/tmp/cert"

    @mock.patch("teradatasql.connect")
    def test_get_sslverifyfull_conn(self, mock_connect):
        extravalues = {"sslmode": "verify-full", "sslca": "/tmp/cert"}
        self.connection.extra = json.dumps(extravalues)
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"
        assert kwargs["sslmode"] == "verify-full"
        assert kwargs["sslca"] == "/tmp/cert"

    @mock.patch("teradatasql.connect")
    def test_get_sslcrc_conn(self, mock_connect):
        extravalues = {"sslcrc": "sslcrc"}
        self.connection.extra = json.dumps(extravalues)
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"
        assert kwargs["sslcrc"] == "sslcrc"

    @mock.patch("teradatasql.connect")
    def test_get_sslprotocol_conn(self, mock_connect):
        extravalues = {"sslprotocol": "protocol"}
        self.connection.extra = json.dumps(extravalues)
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"
        assert kwargs["sslprotocol"] == "protocol"

    @mock.patch("teradatasql.connect")
    def test_get_sslcipher_conn(self, mock_connect):
        extravalues = {"sslcipher": "cipher"}
        self.connection.extra = json.dumps(extravalues)
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"
        assert kwargs["sslcipher"] == "cipher"

    @mock.patch("sqlalchemy.create_engine")
    def test_get_sqlalchemy_conn(self, mock_connect):
        self.db_hook.get_sqlalchemy_engine()
        assert mock_connect.call_count == 1
        args = mock_connect.call_args.args
        assert len(args) == 1
        expected_link = (
            f"teradatasql://{self.connection.login}:{self.connection.password}@{self.connection.host}"
        )
        assert expected_link == args[0]

    def test_get_uri(self):
        ret_uri = self.db_hook.get_uri()
        expected_uri = f"teradata://{self.connection.login}:{self.connection.password}@{self.connection.host}/{self.connection.schema}"
        assert expected_uri == ret_uri

    def test_get_records(self):
        sql = "SQL"
        self.test_db_hook.get_records(sql)
        self.cur.execute.assert_called_once_with(sql)
        assert self.conn.commit.called

    def test_run_without_parameters(self):
        sql = "SQL"
        self.test_db_hook.run(sql)
        self.cur.execute.assert_called_once_with(sql)
        assert self.conn.commit.called

    def test_run_with_parameters(self):
        sql = "SQL"
        param = ("p1", "p2")
        self.test_db_hook.run(sql, parameters=param)
        self.cur.execute.assert_called_once_with(sql, param)
        assert self.conn.commit.called

    def test_insert_rows(self):
        rows = [
            (
                "'test_string",
                None,
                datetime(2023, 8, 15),
                1,
                3.14,
                "str",
            )
        ]
        target_fields = [
            "basestring",
            "none",
            "datetime",
            "int",
            "float",
            "str",
        ]
        self.test_db_hook.insert_rows("table", rows, target_fields)
        self.cur.executemany.assert_called_once_with(
            "INSERT INTO table (basestring, none, datetime, int, float, str) VALUES (?,?,?,?,?,?)",
            [("'test_string", None, "2023-08-15T00:00:00", "1", "3.14", "str")],
        )

    def test_call_proc_dict(self):
        parameters = {"a": 1, "b": 2, "c": 3}

        class bindvar(int):
            def getvalue(self):
                return self

        self.cur.fetchall.return_value = {k: bindvar(v) for k, v in parameters.items()}
        result = self.test_db_hook.callproc("proc", True, parameters)
        assert result == parameters

    def test_set_query_band(self):
        query_band_text = "example_query_band_text"
        _handle_user_query_band_text(query_band_text)
        self.test_db_hook.set_query_band(query_band_text, self.conn)
        self.conn.cursor.assert_called_once()

    @mock.patch("teradatasql.connect")
    def test_query_band_not_in_conn_config(self, mock_connect):
        extravalues = {"query_band": "appname=airflow;org=test;"}
        self.connection.extra = json.dumps(extravalues)
        self.db_hook.get_conn()
        assert mock_connect.call_count == 1
        args, kwargs = mock_connect.call_args
        assert args == ()
        assert kwargs["host"] == "host"
        assert kwargs["database"] == "schema"
        assert kwargs["dbs_port"] == "1025"
        assert kwargs["user"] == "login"
        assert kwargs["password"] == "password"
        assert "query_band" not in kwargs


def test_handle_user_query_band_text_invalid():
    query_band_text = _handle_user_query_band_text("invalid_queryband")
    assert query_band_text == "invalid_queryband;org=teradata-internal-telem;appname=airflow;"


def test_handle_user_query_band_text_override_appname():
    query_band_text = _handle_user_query_band_text("appname=test;")
    assert query_band_text == "appname=test_airflow;org=teradata-internal-telem;"


def test_handle_user_query_band_text_append_org():
    query_band_text = _handle_user_query_band_text("appname=airflow;")
    assert query_band_text == "appname=airflow;org=teradata-internal-telem;"


def test_handle_user_query_band_text_user_org():
    query_band_text = _handle_user_query_band_text("appname=airflow;org=test")
    assert query_band_text == "appname=airflow;org=test"


def test_handle_user_query_band_text_none():
    query_band_text = _handle_user_query_band_text(None)
    assert query_band_text == "org=teradata-internal-telem;appname=airflow;"


def test_handle_user_query_band_text_no_appname():
    query_band_text = _handle_user_query_band_text("org=test;")
    assert query_band_text == "org=test;appname=airflow;"


def test_handle_user_query_band_text_no_appname_with_teradata_org():
    query_band_text = _handle_user_query_band_text("org=teradata-internal-telem;")
    assert query_band_text == "org=teradata-internal-telem;appname=airflow;"



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections.abc import Sequence
from textwrap import dedent
from typing import TYPE_CHECKING

from airflow.models import BaseOperator

try:
    from airflow.providers.microsoft.azure.hooks.wasb import WasbHook
except ModuleNotFoundError as e:
    from airflow.exceptions import AirflowOptionalProviderFeatureException

    raise AirflowOptionalProviderFeatureException(e)

from airflow.providers.teradata.hooks.teradata import TeradataHook

if TYPE_CHECKING:
    from airflow.utils.context import Context


class AzureBlobStorageToTeradataOperator(BaseOperator):
    """

    Loads CSV, JSON and Parquet format data from Azure Blob Storage to Teradata.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:AzureBlobStorageToTeradataOperator`

    :param blob_source_key: The URI format specifying the location of the Azure blob object store.(templated)
        The URI format is `/az/YOUR-STORAGE-ACCOUNT.blob.core.windows.net/YOUR-CONTAINER/YOUR-BLOB-LOCATION`.
        Refer to
        https://docs.teradata.com/search/documents?query=native+object+store&sort=last_update&virtual-field=title_only&content-lang=en-US
    :param public_bucket: Specifies whether the provided blob container is public. If the blob container is public,
        it means that anyone can access the objects within it via a URL without requiring authentication.
        If the bucket is private and authentication is not provided, the operator will throw an exception.
    :param azure_conn_id: The Airflow WASB connection used for azure blob credentials.
    :param teradata_table: The name of the teradata table to which the data is transferred.(templated)
    :param teradata_conn_id: The connection ID used to connect to Teradata
        :ref:`Teradata connection <howto/connection:Teradata>`
    :param teradata_authorization_name: The name of Teradata Authorization Database Object,
        is used to control who can access an Azure Blob object store.
        Refer to
        https://docs.teradata.com/r/Enterprise_IntelliFlex_VMware/Teradata-VantageTM-Native-Object-Store-Getting-Started-Guide-17.20/Setting-Up-Access/Controlling-Foreign-Table-Access-with-an-AUTHORIZATION-Object

    Note that ``blob_source_key`` and ``teradata_table`` are
    templated, so you can use variables in them if you wish.
    """

    template_fields: Sequence[str] = ("blob_source_key", "teradata_table")
    ui_color = "#e07c24"

    def __init__(
        self,
        *,
        blob_source_key: str,
        public_bucket: bool = False,
        azure_conn_id: str = "azure_default",
        teradata_table: str,
        teradata_conn_id: str = "teradata_default",
        teradata_authorization_name: str = "",
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.blob_source_key = blob_source_key
        self.public_bucket = public_bucket
        self.azure_conn_id = azure_conn_id
        self.teradata_table = teradata_table
        self.teradata_conn_id = teradata_conn_id
        self.teradata_authorization_name = teradata_authorization_name

    def execute(self, context: Context) -> None:
        self.log.info(
            "transferring data from %s to teradata table %s...", self.blob_source_key, self.teradata_table
        )
        teradata_hook = TeradataHook(teradata_conn_id=self.teradata_conn_id)
        credentials_part = "ACCESS_ID= '' ACCESS_KEY= ''"
        if not self.public_bucket:
            # Accessing data directly from the Azure Blob Storage and creating permanent table inside the
            # database
            if self.teradata_authorization_name:
                credentials_part = f"AUTHORIZATION={self.teradata_authorization_name}"
            else:
                # Obtaining the Azure client ID and Azure secret in order to access a specified Blob container
                azure_hook = WasbHook(wasb_conn_id=self.azure_conn_id)
                conn = azure_hook.get_connection(self.azure_conn_id)
                access_id = conn.login
                access_secret = conn.password
                credentials_part = f"ACCESS_ID= '{access_id}' ACCESS_KEY= '{access_secret}'"
        sql = dedent(f"""
                        CREATE MULTISET TABLE {self.teradata_table} AS
                        (
                            SELECT * FROM (
                                LOCATION = '{self.blob_source_key}'
                                {credentials_part}
                            ) AS d
                        ) WITH DATA
                        """).rstrip()
        try:
            teradata_hook.run(sql, True)
        except Exception as ex:
            self.log.error(str(ex))
            raise
        self.log.info("The transfer of data from Azure Blob to Teradata was successful")



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections.abc import Sequence
from typing import TYPE_CHECKING, ClassVar

from airflow.models import BaseOperator
from airflow.providers.common.sql.operators.sql import SQLExecuteQueryOperator
from airflow.providers.teradata.hooks.teradata import TeradataHook

if TYPE_CHECKING:
    try:
        from airflow.sdk.definitions.context import Context
    except ImportError:
        # TODO: Remove once provider drops support for Airflow 2
        from airflow.utils.context import Context


class TeradataOperator(SQLExecuteQueryOperator):
    """
    General Teradata Operator to execute queries on Teradata Database.

    Executes sql statements in the Teradata SQL Database using Teradata Python SQL Driver

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:TeradataOperator`

    :param sql: the SQL query to be executed as a single string, or a list of str (sql statements)
    :param teradata_conn_id: reference to a predefined database
    :param autocommit: if True, each command is automatically committed.(default value: False)
    :param parameters: (optional) the parameters to render the SQL query with.
    :param schema: The Teradata database to connect to.
    """

    template_fields: Sequence[str] = (
        "sql",
        "parameters",
    )
    template_ext: Sequence[str] = (".sql",)
    template_fields_renderers: ClassVar[dict] = {"sql": "sql"}
    ui_color = "#e07c24"

    def __init__(
        self,
        teradata_conn_id: str = TeradataHook.default_conn_name,
        schema: str | None = None,
        **kwargs,
    ) -> None:
        if schema:
            hook_params = kwargs.pop("hook_params", {})
            kwargs["hook_params"] = {
                "database": schema,
                **hook_params,
            }
        super().__init__(**kwargs)
        self.conn_id = teradata_conn_id


class TeradataStoredProcedureOperator(BaseOperator):
    """
    Executes stored procedure in a specific Teradata database.

    :param procedure: name of stored procedure to call (templated)
    :param teradata_conn_id: The :ref:`Teradata connection id <howto/connection:teradata>`
        reference to a specific Teradata database.
    :param parameters: (optional, templated) the parameters provided in the call

    """

    template_fields: Sequence[str] = (
        "procedure",
        "parameters",
    )
    ui_color = "#ededed"

    def __init__(
        self,
        *,
        procedure: str,
        teradata_conn_id: str = TeradataHook.default_conn_name,
        parameters: dict | list | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.teradata_conn_id = teradata_conn_id
        self.procedure = procedure
        self.parameters = parameters

    def execute(self, context: Context):
        hook = TeradataHook(teradata_conn_id=self.teradata_conn_id)
        return hook.callproc(self.procedure, autocommit=True, parameters=self.parameters)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example use of Teradata Compute Cluster Provision Operator
"""

from __future__ import annotations

import datetime
import os

import pytest

from airflow import DAG
from airflow.models import Param

try:
    from airflow.providers.teradata.operators.teradata_compute_cluster import (
        TeradataComputeClusterDecommissionOperator,
        TeradataComputeClusterProvisionOperator,
        TeradataComputeClusterResumeOperator,
        TeradataComputeClusterSuspendOperator,
    )
except ImportError:
    pytest.skip("TERADATA provider not available", allow_module_level=True)

# [START teradata_vantage_lake_compute_cluster_howto_guide]


ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "example_teradata_computer_cluster"

with DAG(
    dag_id=DAG_ID,
    start_date=datetime.datetime(2020, 2, 2),
    schedule="@once",
    catchup=False,
    default_args={"teradata_conn_id": "teradata_lake"},
    render_template_as_native_obj=True,
    params={
        "compute_group_name": Param(
            "compute_group_test",
            type="string",
            title="Compute cluster group Name:",
            description="Enter compute cluster group name.",
        ),
        "compute_profile_name": Param(
            "compute_profile_test",
            type="string",
            title="Compute cluster profile Name:",
            description="Enter compute cluster profile name.",
        ),
        "query_strategy": Param(
            "STANDARD",
            type="string",
            title="Compute cluster instance type:",
            description="Enter compute cluster instance type. Valid values are STANDARD, ANALYTIC",
        ),
        "compute_map": Param(
            "TD_COMPUTE_XSMALL",
            type="string",
            title="Compute Map Name:",
            description="Enter compute cluster compute map name.",
        ),
        "compute_attribute": Param(
            "MIN_COMPUTE_COUNT(1) MAX_COMPUTE_COUNT(5) INITIALLY_SUSPENDED('FALSE')",
            type="string",
            title="Compute cluster compute attribute:",
            description="Enter compute cluster compute attribute values.",
        ),
        "teradata_conn_id": Param(
            "teradata_lake",
            type="string",
            title="Teradata ConnectionId:",
            description="Enter Teradata connection id.",
        ),
        "timeout": Param(
            20,
            type="integer",
            title="Timeout:",
            description="Time elapsed before the task times out and fails. Timeout is in minutes.",
        ),
    },
) as dag:
    # [START teradata_vantage_lake_compute_cluster_provision_howto_guide]
    compute_cluster_provision_operation = TeradataComputeClusterProvisionOperator(
        task_id="compute_cluster_provision_operation",
        compute_profile_name="{{ params.compute_profile_name }}",
        compute_group_name="{{ params.compute_group_name }}",
        teradata_conn_id="{{ params.teradata_conn_id }}",
        timeout="{{ params.timeout }}",
        query_strategy="{{ params.query_strategy }}",
        compute_map="{{ params.compute_map }}",
        compute_attribute="{{ params.compute_attribute }}",
    )
    # [END teradata_vantage_lake_compute_cluster_provision_howto_guide]
    # [START teradata_vantage_lake_compute_cluster_suspend_howto_guide]
    compute_cluster_suspend_operation = TeradataComputeClusterSuspendOperator(
        task_id="compute_cluster_suspend_operation",
        compute_profile_name="{{ params.compute_profile_name }}",
        compute_group_name="{{ params.compute_group_name }}",
        teradata_conn_id="{{ params.teradata_conn_id }}",
        timeout="{{ params.timeout }}",
    )
    # [END teradata_vantage_lake_compute_cluster_suspend_howto_guide]
    # [START teradata_vantage_lake_compute_cluster_resume_howto_guide]
    compute_cluster_resume_operation = TeradataComputeClusterResumeOperator(
        task_id="compute_cluster_resume_operation",
        compute_profile_name="{{ params.compute_profile_name }}",
        compute_group_name="{{ params.compute_group_name }}",
        teradata_conn_id="{{ params.teradata_conn_id }}",
        timeout="{{ params.timeout }}",
    )
    # [END teradata_vantage_lake_compute_cluster_resume_howto_guide]
    # [START teradata_vantage_lake_compute_cluster_decommission_howto_guide]
    compute_cluster_decommission_operation = TeradataComputeClusterDecommissionOperator(
        task_id="compute_cluster_decommission_operation",
        compute_profile_name="{{ params.compute_profile_name }}",
        compute_group_name="{{ params.compute_group_name }}",
        delete_compute_group=bool("{{ params.delete_compute_group }}"),
        teradata_conn_id="{{ params.teradata_conn_id }}",
        timeout="{{ params.timeout }}",
    )
    # [END teradata_vantage_lake_compute_cluster_decommission_howto_guide]
    (
        compute_cluster_provision_operation
        >> compute_cluster_suspend_operation
        >> compute_cluster_resume_operation
        >> compute_cluster_decommission_operation
    )

    # [END teradata_vantage_lake_compute_cluster_howto_guide]

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG to show usage of S3StorageToTeradataOperator.

The transfer operator moves files in CSV, JSON, and PARQUET formats from S3
to Teradata tables. In the example Directed Acyclic Graph (DAG) below, it assumes Airflow
Connections with the IDs `teradata_default` and `aws_default` already exist. The DAG creates
tables using data from the S3, reports the number of rows inserted into
the table, and subsequently drops the table.
"""

from __future__ import annotations

import datetime
import os

import pytest

from airflow import DAG
from airflow.providers.teradata.transfers.s3_to_teradata import S3ToTeradataOperator

try:
    from airflow.providers.teradata.operators.teradata import TeradataOperator
except ImportError:
    pytest.skip("Teradata provider apache-airflow-provider-teradata not available", allow_module_level=True)

# [START s3_to_teradata_transfer_operator_howto_guide]


ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "example_s3_to_teradata_transfer_operator"
CONN_ID = "teradata_default"

with DAG(
    dag_id=DAG_ID,
    start_date=datetime.datetime(2020, 2, 2),
    schedule="@once",
    catchup=False,
    default_args={"teradata_conn_id": CONN_ID},
) as dag:
    # [START s3_to_teradata_transfer_operator_howto_guide_transfer_data_public_s3_to_teradata_csv]
    transfer_data_csv = S3ToTeradataOperator(
        task_id="transfer_data_s3_to_teradata_csv",
        s3_source_key="/s3/td-usgs-public.s3.amazonaws.com/CSVDATA/09394500/2018/06/",
        public_bucket=True,
        teradata_table="example_s3_teradata_csv",
        aws_conn_id="aws_default",
        trigger_rule="all_done",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_transfer_data_public_s3_to_teradata_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    read_data_table_csv = TeradataOperator(
        task_id="read_data_table_csv",
        conn_id=CONN_ID,
        sql="SELECT * from example_s3_teradata_csv;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    drop_table_csv = TeradataOperator(
        task_id="drop_table_csv",
        conn_id=CONN_ID,
        sql="DROP TABLE example_s3_teradata_csv;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_transfer_data_access_s3_to_teradata_csv]
    transfer_key_data_csv = S3ToTeradataOperator(
        task_id="transfer_key_data_s3_to_teradata_key_csv",
        s3_source_key="/s3/airflowteradatatest.s3.ap-southeast-2.amazonaws.com/",
        teradata_table="example_s3_teradata_csv",
        aws_conn_id="aws_default",
        teradata_conn_id="teradata_default",
        trigger_rule="all_done",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_transfer_data_access_s3_to_teradata_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    read_key_data_table_csv = TeradataOperator(
        task_id="read_key_data_table_csv",
        conn_id=CONN_ID,
        sql="SELECT * from example_s3_teradata_csv;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    drop_key_table_csv = TeradataOperator(
        task_id="drop_key_table_csv",
        conn_id=CONN_ID,
        sql="DROP TABLE example_s3_teradata_csv;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_create_authorization]
    create_aws_authorization = TeradataOperator(
        task_id="create_aws_authorization",
        conn_id=CONN_ID,
        sql="CREATE AUTHORIZATION aws_authorization USER '{{ var.value.get('AWS_ACCESS_KEY_ID') }}' PASSWORD '{{ var.value.get('AWS_SECRET_ACCESS_KEY') }}' ",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_create_authorization]
    # [START s3_to_teradata_transfer_operator_howto_guide_transfer_data_authorization_s3_to_teradata_csv]
    transfer_auth_data_csv = S3ToTeradataOperator(
        task_id="transfer_auth_data_s3_to_teradata_auth_csv",
        s3_source_key="/s3/teradata-download.s3.us-east-1.amazonaws.com/DevTools/csv/",
        teradata_table="example_s3_teradata_csv",
        teradata_authorization_name="aws_authorization",
        teradata_conn_id="teradata_default",
        trigger_rule="all_done",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_transfer_data_authorization_s3_to_teradata_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    read_auth_data_table_csv = TeradataOperator(
        task_id="read_auth_data_table_csv",
        conn_id=CONN_ID,
        sql="SELECT * from example_s3_teradata_csv;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    drop_auth_table_csv = TeradataOperator(
        task_id="drop_auth_table_csv",
        conn_id=CONN_ID,
        sql="DROP TABLE example_s3_teradata_csv;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    # [START s3_to_teradata_transfer_operator_howto_guide_drop_authorization]
    drop_auth = TeradataOperator(
        task_id="drop_auth",
        conn_id=CONN_ID,
        sql="DROP AUTHORIZATION aws_authorization;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_drop_authorization]
    # [START s3_to_teradata_transfer_operator_howto_guide_transfer_data_s3_to_teradata_json]
    transfer_data_json = S3ToTeradataOperator(
        task_id="transfer_data_s3_to_teradata_json",
        s3_source_key="/s3/td-usgs-public.s3.amazonaws.com/JSONDATA/09394500/2018/06/",
        public_bucket=True,
        teradata_table="example_s3_teradata_json",
        aws_conn_id="aws_default",
        trigger_rule="all_done",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_transfer_data_s3_to_teradata_json]
    # [START s3_to_teradata_transfer_operator_howto_guide_read_data_table_json]
    read_data_table_json = TeradataOperator(
        task_id="read_data_table_json",
        sql="SELECT * from example_s3_teradata_json;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_read_data_table_json]
    # [START s3_to_teradata_transfer_operator_howto_guide_drop_table_json]
    drop_table_json = TeradataOperator(
        task_id="drop_table_json",
        sql="DROP TABLE example_s3_teradata_json;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_drop_table_json]
    # [START s3_to_teradata_transfer_operator_howto_guide_transfer_data_s3_to_teradata_parquet]
    transfer_data_parquet = S3ToTeradataOperator(
        task_id="transfer_data_s3_to_teradata_parquet",
        s3_source_key="/s3/td-usgs-public.s3.amazonaws.com/PARQUETDATA/09394500/2018/06/",
        public_bucket=True,
        teradata_table="example_s3_teradata_parquet",
        aws_conn_id="aws_default",
        trigger_rule="all_done",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_transfer_data_s3_to_teradata_parquet]
    # [START s3_to_teradata_transfer_operator_howto_guide_read_data_table_parquet]
    read_data_table_parquet = TeradataOperator(
        task_id="read_data_table_parquet",
        sql="SELECT * from example_s3_teradata_parquet;",
    )
    # [END s3_to_teradata_transfer_operator_howto_guide_read_data_table_parquet]
    # [START s3_to_teradata_transfer_operator_howto_guide_drop_table]
    drop_table_parquet = TeradataOperator(
        task_id="drop_table_parquet",
        sql="DROP TABLE example_s3_teradata_parquet;",
    )

    # [END s3_to_teradata_transfer_operator_howto_guide_drop_table]
    (
        transfer_data_csv
        >> transfer_data_json
        >> transfer_data_parquet
        >> read_data_table_csv
        >> read_data_table_json
        >> read_data_table_parquet
        >> drop_table_csv
        >> drop_table_json
        >> drop_table_parquet
        >> transfer_key_data_csv
        >> read_key_data_table_csv
        >> drop_key_table_csv
        >> create_aws_authorization
        >> transfer_auth_data_csv
        >> read_auth_data_table_csv
        >> drop_auth_table_csv
        >> drop_auth
    )
    # [END s3_to_teradata_transfer_operator_howto_guide]

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG to show usage of TeradataOperator with SSL teradata connection.

This DAG assumes Airflow Connection with connection id `teradata_ssl_default` already exists in locally. It
shows how to use create, update, delete and select teradata statements with TeradataOperator as tasks in
airflow dags using TeradataStoredProcedureOperator.
"""

from __future__ import annotations

import datetime
import os

import pytest

from airflow import DAG

try:
    from airflow.providers.teradata.operators.teradata import TeradataOperator
except ImportError:
    pytest.skip("TERADATA provider not available", allow_module_level=True)

# [START teradata_operator_howto_guide]


# create_table_teradata, insert_teradata_task, create_table_teradata_from_external_file,
# populate_user_table get_all_countries, get_all_description, get_countries_from_continent,
# drop_table_teradata_task, drop_users_table_teradata_task are examples of tasks created by instantiating
# the Teradata Operator

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "example_ssl_teradata"
CONN_ID = "teradata_ssl_default"

with DAG(
    dag_id=DAG_ID,
    start_date=datetime.datetime(2020, 2, 2),
    schedule="@once",
    catchup=False,
    default_args={"teradata_conn_id": CONN_ID},
) as dag:
    # [START teradata_operator_howto_guide_create_country_table]
    create_country_table = TeradataOperator(
        task_id="create_country_table",
        sql=r"""
        CREATE TABLE SSL_Country (
            country_id INTEGER,
            name CHAR(25),
            continent CHAR(25)
        );
        """,
        trigger_rule="all_done",
    )
    # [END teradata_operator_howto_guide_create_country_table]
    # [START teradata_operator_howto_guide_populate_country_table]
    populate_country_table = TeradataOperator(
        task_id="populate_country_table",
        sql=r"""
                INSERT INTO SSL_Country VALUES ( 1, 'India', 'Asia');
                INSERT INTO SSL_Country VALUES ( 2, 'Germany', 'Europe');
                INSERT INTO SSL_Country VALUES ( 3, 'Argentina', 'South America');
                INSERT INTO SSL_Country VALUES ( 4, 'Ghana', 'Africa');
                """,
    )
    # [END teradata_operator_howto_guide_populate_country_table]
    # [START teradata_operator_howto_guide_create_users_table_from_external_file]
    create_users_table_from_external_file = TeradataOperator(
        task_id="create_users_table_from_external_file",
        sql="create_ssl_table.sql",
        dag=dag,
    )
    # [END teradata_operator_howto_guide_create_users_table_from_external_file]
    # [START teradata_operator_howto_guide_get_all_countries]
    get_all_countries = TeradataOperator(
        task_id="get_all_countries",
        sql=r"SELECT * FROM SSL_Country;",
    )
    # [END teradata_operator_howto_guide_get_all_countries]
    # [START teradata_operator_howto_guide_params_passing_get_query]
    get_countries_from_continent = TeradataOperator(
        task_id="get_countries_from_continent",
        sql=r"SELECT * FROM SSL_Country where {{ params.column }}='{{ params.value }}';",
        params={"column": "continent", "value": "Asia"},
    )
    # [END teradata_operator_howto_guide_params_passing_get_query]
    # [START teradata_operator_howto_guide_drop_country_table]
    drop_country_table = TeradataOperator(
        task_id="drop_country_table", sql=r"DROP TABLE SSL_Country;", dag=dag
    )
    # [END teradata_operator_howto_guide_drop_country_table]
    # [START teradata_operator_howto_guide_drop_users_table]
    drop_users_table = TeradataOperator(task_id="drop_users_table", sql=r"DROP TABLE SSL_Users;", dag=dag)
    # [END teradata_operator_howto_guide_drop_users_table]

    (
        create_country_table
        >> populate_country_table
        >> create_users_table_from_external_file
        >> get_all_countries
        >> get_countries_from_continent
        >> drop_country_table
        >> drop_users_table
    )

    # [END teradata_operator_howto_guide]

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from airflow.providers.teradata.utils.constants import Constants


def test_create_operations():
    assert Constants.CC_CREATE_OPR == "CREATE"


def test_create_suspend_operations():
    assert Constants.CC_CREATE_SUSPEND_OPR == "CREATE_SUSPEND"


def test_drop_operations():
    assert Constants.CC_DROP_OPR == "DROP"


def test_suspend_operations():
    assert Constants.CC_SUSPEND_OPR == "SUSPEND"


def test_resume_operations():
    assert Constants.CC_RESUME_OPR == "RESUME"


def test_initialize_db_status():
    assert Constants.CC_INITIALIZE_DB_STATUS == "Initializing"


def test_suspend_db_status():
    assert Constants.CC_SUSPEND_DB_STATUS == "Suspended"


def test_resume_db_status():
    assert Constants.CC_RESUME_DB_STATUS == "Running"


def test_operation_success_message():
    expected_msg = "Compute Cluster %s  %s operation completed successfully."
    assert expected_msg == Constants.CC_OPR_SUCCESS_STATUS_MSG


def test_operation_failure_message():
    expected_msg = "Compute Cluster %s  %s operation has failed."
    assert expected_msg == Constants.CC_OPR_FAILURE_STATUS_MSG


def test_initializing_status_message():
    expected_msg = "The environment is currently initializing. Please wait."
    assert expected_msg == Constants.CC_OPR_INITIALIZING_STATUS_MSG


def test_empty_profile_error_message():
    expected_msg = "Please provide a valid name for the compute cluster profile."
    assert expected_msg == Constants.CC_OPR_EMPTY_PROFILE_ERROR_MSG


def test_non_exists_message():
    expected_msg = "The specified Compute cluster is not present or The user doesn't have permission to access compute cluster."
    assert expected_msg == Constants.CC_GRP_PRP_NON_EXISTS_MSG


def test_unauthorized_message():
    expected_msg = "The %s operation is not authorized for the user."
    assert expected_msg == Constants.CC_GRP_PRP_UN_AUTHORIZED_MSG


def test_lake_support_only_message():
    expected_msg = "Compute Groups is supported only on Vantage Cloud Lake."
    assert expected_msg == Constants.CC_GRP_LAKE_SUPPORT_ONLY_MSG


def test_timeout_error_message():
    expected_msg = "There is an issue with the %s operation. Kindly consult the administrator for assistance."
    assert expected_msg == Constants.CC_OPR_TIMEOUT_ERROR


def test_exists_message():
    expected_msg = "The specified Compute cluster is already exists."
    assert expected_msg == Constants.CC_GRP_PRP_EXISTS_MSG


def test_empty_copy_profile_error_message():
    expected_msg = "Please provide a valid name for the source and target compute profile."
    assert expected_msg == Constants.CC_OPR_EMPTY_COPY_PROFILE_ERROR_MSG


def test_timeout_value():
    assert Constants.CC_OPR_TIME_OUT == 1200


def test_poll_interval():
    assert Constants.CC_POLL_INTERVAL == 60



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG to show usage of TeradataOperator.

This DAG assumes Airflow Connection with connection id `teradata_default` already exists in locally. It
shows how to use create, update, delete and select teradata statements with TeradataOperator as tasks in
airflow dags using TeradataStoredProcedureOperator.
"""

from __future__ import annotations

import datetime
import os

import pytest

from airflow import DAG

try:
    from airflow.providers.teradata.operators.teradata import TeradataOperator
except ImportError:
    pytest.skip("TERADATA provider not available", allow_module_level=True)

# [START teradata_operator_howto_guide]


ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "example_teradata"
CONN_ID = "teradata_default"

with DAG(
    dag_id=DAG_ID,
    start_date=datetime.datetime(2020, 2, 2),
    schedule="@once",
    catchup=False,
    default_args={"teradata_conn_id": CONN_ID},
) as dag:
    # [START teradata_operator_howto_guide_create_table]
    create_table = TeradataOperator(
        task_id="create_table",
        sql=r"""
        CREATE TABLE Country (
            country_id INTEGER,
            name CHAR(25),
            continent CHAR(25)
        );
        """,
    )
    # [END teradata_operator_howto_guide_create_table]

    # [START teradata_operator_howto_guide_create_table_from_external_file]
    create_table_from_external_file = TeradataOperator(
        task_id="create_table_from_external_file",
        sql="create_table.sql",
        dag=dag,
    )
    # [END teradata_operator_howto_guide_create_table_from_external_file]
    # [START teradata_operator_howto_guide_populate_table]
    populate_table = TeradataOperator(
        task_id="populate_table",
        sql=r"""
        INSERT INTO Users (username, description)
            VALUES ( 'Danny', 'Musician');
        INSERT INTO Users (username, description)
            VALUES ( 'Simone', 'Chef');
        INSERT INTO Users (username, description)
            VALUES ( 'Lily', 'Florist');
        INSERT INTO Users (username, description)
            VALUES ( 'Tim', 'Pet shop owner');
        """,
    )
    # [END teradata_operator_howto_guide_populate_table]
    # [START teradata_operator_howto_guide_get_all_countries]
    get_all_countries = TeradataOperator(
        task_id="get_all_countries",
        sql=r"SELECT * FROM Country;",
    )
    # [END teradata_operator_howto_guide_get_all_countries]
    # [START teradata_operator_howto_guide_params_passing_get_query]
    get_countries_from_continent = TeradataOperator(
        task_id="get_countries_from_continent",
        sql=r"SELECT * FROM Country WHERE {{ params.column }}='{{ params.value }}';",
        params={"column": "continent", "value": "Asia"},
    )
    # [END teradata_operator_howto_guide_params_passing_get_query]
    # [START teradata_operator_howto_guide_drop_country_table]
    drop_country_table = TeradataOperator(
        task_id="drop_country_table",
        sql=r"DROP TABLE Country;",
        dag=dag,
    )
    # [END teradata_operator_howto_guide_drop_country_table]
    # [START teradata_operator_howto_guide_drop_users_table]
    drop_users_table = TeradataOperator(
        task_id="drop_users_table",
        sql=r"DROP TABLE Users;",
        dag=dag,
    )
    # [END teradata_operator_howto_guide_drop_users_table]
    # [START teradata_operator_howto_guide_create_schema]
    create_schema = TeradataOperator(
        task_id="create_schema",
        sql=r"CREATE DATABASE airflow_temp AS PERM=10e6;",
    )
    # [END teradata_operator_howto_guide_create_schema]
    # [START teradata_operator_howto_guide_create_table_with_schema]
    create_table_with_schema = TeradataOperator(
        task_id="create_table_with_schema",
        sql=r"""
        CREATE TABLE schema_table (
           country_id INTEGER,
           name CHAR(25),
           continent CHAR(25)
        );
        """,
        schema="airflow_temp",
    )
    # [END teradata_operator_howto_guide_create_table_with_schema]
    # [START teradata_operator_howto_guide_drop_schema_table]
    drop_schema_table = TeradataOperator(
        task_id="drop_schema_table",
        sql=r"DROP TABLE schema_table;",
        dag=dag,
        schema="airflow_temp",
    )
    # [END teradata_operator_howto_guide_drop_schema_table]
    # [START teradata_operator_howto_guide_drop_schema]
    drop_schema = TeradataOperator(
        task_id="drop_schema",
        sql=r"DROP DATABASE airflow_temp;",
        dag=dag,
    )

    # [END teradata_operator_howto_guide_drop_schema]
    (
        create_table
        >> create_table_from_external_file
        >> populate_table
        >> get_all_countries
        >> get_countries_from_continent
        >> drop_country_table
        >> drop_users_table
        >> create_schema
        >> create_table_with_schema
        >> drop_schema_table
        >> drop_schema
    )

    # [END teradata_operator_howto_guide]

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from decimal import Decimal
from unittest import mock

import pytest

from airflow.providers.teradata.hooks.teradata import TeradataHook
from airflow.providers.teradata.transfers.teradata_to_teradata import TeradataToTeradataOperator


@pytest.fixture
def mocked_src_hook():
    with mock.patch.object(
        TeradataToTeradataOperator, "src_hook", spec=TeradataHook, name="TeradataSourceHook"
    ) as m:
        yield m


@pytest.fixture
def mocked_dest_hook():
    with mock.patch.object(
        TeradataToTeradataOperator, "dest_hook", spec=TeradataHook, name="TeradataDestHook"
    ) as m:
        yield m


class TestTeradataToTeradataTransfer:
    dest_teradata_conn_id = "dest_teradata_conn_id"
    destination_table = "destination_table"
    source_teradata_conn_id = "source_teradata_conn_id"
    sql = (r"""select DATE where DATE > {{ sql_params.ref_date }}   ;""",)
    sql_params = {"ref_date": "2018-01-01"}

    def test_source_hook(self):
        op = TeradataToTeradataOperator(
            task_id="transfer_data",
            dest_teradata_conn_id=self.dest_teradata_conn_id,
            destination_table=self.destination_table,
            source_teradata_conn_id=self.source_teradata_conn_id,
            sql=self.sql,
            sql_params=self.sql_params,
        )
        hook = op.src_hook
        assert hook
        assert hook is op.src_hook
        assert hook.teradata_conn_id == "source_teradata_conn_id"

    def test_destination_hook(self):
        op = TeradataToTeradataOperator(
            task_id="transfer_data",
            dest_teradata_conn_id=self.dest_teradata_conn_id,
            destination_table=self.destination_table,
            source_teradata_conn_id=self.source_teradata_conn_id,
            sql=self.sql,
            sql_params=self.sql_params,
        )
        hook = op.dest_hook
        assert hook
        assert hook is op.dest_hook
        assert hook.teradata_conn_id == "dest_teradata_conn_id"

    def test_execution(self, mocked_src_hook, mocked_dest_hook):
        cursor_description = [
            ["user_id", Decimal, None, 8, 10, 0, False],
            ["user_name", str, None, 60, None, None, True],
        ]

        cursor_rows = [[Decimal("1"), "User1"], [Decimal("2"), "User2"], [Decimal("3"), "User3"]]

        mock_src_conn = mocked_src_hook.get_conn.return_value.__enter__.return_value
        mock_cursor = mock_src_conn.cursor.return_value
        mock_cursor.description.__iter__.return_value = cursor_description
        mock_cursor.fetchmany.side_effect = [cursor_rows, []]
        rows_chunk = 5000

        op = TeradataToTeradataOperator(
            task_id="transfer_data",
            dest_teradata_conn_id=self.dest_teradata_conn_id,
            destination_table=self.destination_table,
            source_teradata_conn_id=self.source_teradata_conn_id,
            sql=self.sql,
            sql_params=self.sql_params,
        )
        op.execute({})

        assert mocked_src_hook.get_conn.called
        assert mock_src_conn.cursor.called
        mock_cursor.execute.assert_called_once_with(self.sql, self.sql_params)

        calls = [
            mock.call(rows_chunk),
        ]
        mock_cursor.fetchmany.assert_has_calls(calls)
        mocked_dest_hook.insert_rows.assert_called_once_with(
            self.destination_table,
            cursor_rows,
            commit_every=rows_chunk,
            target_fields=["user_id", "user_name"],
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest.mock import call, patch

import pytest

from airflow.providers.common.sql.hooks.handlers import fetch_one_handler
from airflow.providers.teradata.hooks.teradata import TeradataHook
from airflow.providers.teradata.triggers.teradata_compute_cluster import TeradataComputeClusterSyncTrigger
from airflow.providers.teradata.utils.constants import Constants
from airflow.triggers.base import TriggerEvent


@pytest.mark.asyncio
async def test_run_suspend_success():
    trigger = TeradataComputeClusterSyncTrigger(
        teradata_conn_id="test_conn_id",
        compute_profile_name="test_profile",
        operation_type=Constants.CC_SUSPEND_OPR,
        poll_interval=1,
    )
    with patch.object(trigger, "get_status") as mock_get_status:
        mock_get_status.return_value = Constants.CC_SUSPEND_DB_STATUS
        async for event in trigger.run():
            assert event == TriggerEvent(
                {
                    "status": "success",
                    "message": Constants.CC_OPR_SUCCESS_STATUS_MSG
                    % ("test_profile", Constants.CC_SUSPEND_OPR),
                }
            )
        mock_get_status.assert_called_once()


@pytest.mark.asyncio
async def test_run_suspend_success_cg():
    trigger = TeradataComputeClusterSyncTrigger(
        teradata_conn_id="test_conn_id",
        compute_profile_name="test_profile",
        operation_type=Constants.CC_SUSPEND_OPR,
        poll_interval=1,
    )
    with patch.object(trigger, "get_status") as mock_get_status:
        mock_get_status.return_value = Constants.CC_SUSPEND_DB_STATUS
        async for event in trigger.run():
            assert event == TriggerEvent(
                {
                    "status": "success",
                    "message": Constants.CC_OPR_SUCCESS_STATUS_MSG
                    % ("test_profile", Constants.CC_SUSPEND_OPR),
                }
            )
        mock_get_status.assert_called_once()


@pytest.mark.asyncio
async def test_run_suspend_failure():
    trigger = TeradataComputeClusterSyncTrigger(
        teradata_conn_id="test_conn_id",
        compute_profile_name="test_profile",
        operation_type=Constants.CC_SUSPEND_OPR,
        poll_interval=1,
    )
    with patch.object(trigger, "get_status") as mock_get_status:
        mock_get_status.return_value = None
        async for event in trigger.run():
            assert event == TriggerEvent({"status": "error", "message": Constants.CC_GRP_PRP_NON_EXISTS_MSG})
        mock_get_status.assert_called_once()


@pytest.mark.asyncio
async def test_run_resume_success():
    trigger = TeradataComputeClusterSyncTrigger(
        teradata_conn_id="test_conn_id",
        compute_profile_name="test_profile",
        operation_type=Constants.CC_RESUME_OPR,
        poll_interval=1,
    )
    with patch.object(trigger, "get_status") as mock_get_status:
        mock_get_status.return_value = Constants.CC_RESUME_DB_STATUS
        async for event in trigger.run():
            assert event == TriggerEvent(
                {
                    "status": "success",
                    "message": Constants.CC_OPR_SUCCESS_STATUS_MSG
                    % ("test_profile", Constants.CC_RESUME_OPR),
                }
            )
        mock_get_status.assert_called_once()


@pytest.mark.asyncio
async def test_run_resume_failure():
    trigger = TeradataComputeClusterSyncTrigger(
        teradata_conn_id="test_conn_id",
        compute_profile_name="test_profile",
        operation_type=Constants.CC_RESUME_OPR,
        poll_interval=1,
    )
    with patch.object(trigger, "get_status") as mock_get_status:
        mock_get_status.return_value = None
        async for event in trigger.run():
            assert event == TriggerEvent({"status": "error", "message": Constants.CC_GRP_PRP_NON_EXISTS_MSG})
        mock_get_status.assert_called_once()


@pytest.fixture
def mock_teradata_hook_run():
    with patch.object(TeradataHook, "run") as mock_run:
        yield mock_run


@pytest.mark.asyncio
async def test_get_status(mock_teradata_hook_run):
    trigger = TeradataComputeClusterSyncTrigger(
        teradata_conn_id="test_conn_id",
        compute_profile_name="test_profile",
        operation_type=Constants.CC_SUSPEND_OPR,
        poll_interval=1,
    )
    mock_teradata_hook_run.return_value = [Constants.CC_SUSPEND_DB_STATUS]
    status = await trigger.get_status()
    assert status == Constants.CC_SUSPEND_DB_STATUS
    mock_teradata_hook_run.assert_called_once()
    mock_teradata_hook_run.assert_has_calls(
        [
            call(
                "SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = "
                "UPPER('test_profile')",
                handler=fetch_one_handler,
            ),
        ]
    )


@pytest.mark.asyncio
async def test_get_status_cg(mock_teradata_hook_run):
    trigger = TeradataComputeClusterSyncTrigger(
        teradata_conn_id="test_conn_id",
        compute_profile_name="test_profile",
        compute_group_name="test_group",
        operation_type=Constants.CC_RESUME_OPR,
        poll_interval=1,
    )
    mock_teradata_hook_run.return_value = [Constants.CC_RESUME_DB_STATUS]
    status = await trigger.get_status()
    assert status == Constants.CC_RESUME_DB_STATUS
    mock_teradata_hook_run.assert_called_once()
    mock_teradata_hook_run.assert_has_calls(
        [
            call(
                "SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = "
                "UPPER('test_profile') AND UPPER(ComputeGroupName) = UPPER('test_group')",
                handler=fetch_one_handler,
            ),
        ]
    )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock
from unittest.mock import MagicMock, Mock

from airflow.exceptions import AirflowException
from airflow.providers.common.sql.hooks.handlers import fetch_all_handler
from airflow.providers.teradata.hooks.teradata import TeradataHook
from airflow.providers.teradata.operators.teradata import TeradataOperator, TeradataStoredProcedureOperator


class TestTeradataOperator:
    @mock.patch("airflow.providers.common.sql.operators.sql.SQLExecuteQueryOperator.get_db_hook")
    def test_get_hook_from_conn(self, mock_get_db_hook):
        """
        :class:`~.TeradataOperator` should use the hook returned by :meth:`airflow.models.Connection.get_hook`
        if one is returned.

        Specifically we verify here that :meth:`~.TeradataOperator.get_hook` returns the hook returned from a
        call of ``get_hook`` on the object returned from :meth:`~.BaseHook.get_connection`.
        """
        mock_hook = MagicMock()
        mock_get_db_hook.return_value = mock_hook

        op = TeradataOperator(task_id="test", sql="")
        assert op.get_db_hook() == mock_hook

    @mock.patch(
        "airflow.providers.common.sql.operators.sql.SQLExecuteQueryOperator.get_db_hook",
        autospec=TeradataHook,
    )
    def test_get_hook_default(self, mock_get_db_hook):
        """
        If :meth:`airflow.models.Connection.get_hook` does not return a hook (e.g. because of an invalid
        conn type), then :class:`~.TeradataHook` should be used.
        """
        mock_get_db_hook.return_value.side_effect = Mock(side_effect=AirflowException())

        op = TeradataOperator(task_id="test", sql="")
        assert op.get_db_hook().__class__.__name__ == "TeradataHook"

    @mock.patch("airflow.providers.common.sql.operators.sql.SQLExecuteQueryOperator.get_db_hook")
    def test_execute(self, mock_get_db_hook):
        sql = "SELECT * FROM test_table"
        teradata_conn_id = "teradata_default"
        parameters = {"parameter": "value"}
        autocommit = False
        context = "test_context"
        task_id = "test_task_id"

        operator = TeradataOperator(
            sql=sql, teradata_conn_id=teradata_conn_id, parameters=parameters, task_id=task_id
        )
        operator.execute(context=context)
        mock_get_db_hook.return_value.run.assert_called_once_with(
            sql=sql,
            autocommit=autocommit,
            parameters=parameters,
            handler=fetch_all_handler,
            return_last=True,
        )

    @mock.patch("airflow.providers.common.sql.operators.sql.SQLExecuteQueryOperator.get_db_hook")
    def test_teradata_operator_test_multi(self, mock_get_db_hook):
        sql = [
            "CREATE TABLE IF NOT EXISTS test_airflow (dummy VARCHAR(50))",
            "TRUNCATE TABLE test_airflow",
            "INSERT INTO test_airflow VALUES ('X')",
        ]
        teradata_conn_id = "teradata_default"
        parameters = {"parameter": "value"}
        autocommit = False
        context = "test_context"
        task_id = "test_task_id"

        operator = TeradataOperator(
            sql=sql, teradata_conn_id=teradata_conn_id, parameters=parameters, task_id=task_id
        )
        operator.execute(context=context)
        mock_get_db_hook.return_value.run.assert_called_once_with(
            sql=sql,
            autocommit=autocommit,
            parameters=parameters,
            handler=fetch_all_handler,
            return_last=True,
        )


class TestTeradataStoredProcedureOperator:
    @mock.patch.object(TeradataHook, "run", autospec=TeradataHook.run)
    def test_execute(self, mock_run):
        procedure = "test"
        teradata_conn_id = "teradata_default"
        parameters = {"parameter": "value"}
        context = "test_context"
        task_id = "test_task_id"

        operator = TeradataStoredProcedureOperator(
            procedure=procedure,
            teradata_conn_id=teradata_conn_id,
            parameters=parameters,
            task_id=task_id,
        )
        result = operator.execute(context=context)
        assert result is mock_run.return_value
        mock_run.assert_called_once_with(
            mock.ANY,
            "{CALL test(?)}",
            autocommit=True,
            parameters=["value"],
            handler=mock.ANY,
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from datetime import datetime
from unittest import mock

from boto3.session import Session

from airflow.models.connection import Connection
from airflow.providers.teradata.transfers.s3_to_teradata import S3ToTeradataOperator

DEFAULT_DATE = datetime(2024, 1, 1)

AWS_CONN_ID = "aws_default"
TERADATA_CONN_ID = "teradata_default"
S3_SOURCE_KEY = "aws/test"
TERADATA_TABLE = "test"
TASK_ID = "transfer_file"


class TestS3ToTeradataTransfer:
    def test_init(self):
        operator = S3ToTeradataOperator(
            s3_source_key=S3_SOURCE_KEY,
            teradata_table=TERADATA_TABLE,
            aws_conn_id=AWS_CONN_ID,
            teradata_conn_id=TERADATA_CONN_ID,
            task_id=TASK_ID,
            dag=None,
        )

        assert operator.aws_conn_id == AWS_CONN_ID
        assert operator.s3_source_key == S3_SOURCE_KEY
        assert operator.teradata_conn_id == TERADATA_CONN_ID
        assert operator.teradata_table == TERADATA_TABLE
        assert operator.task_id == TASK_ID

    @mock.patch("airflow.providers.amazon.aws.hooks.s3.S3Hook.get_connection")
    @mock.patch("airflow.models.connection.Connection")
    @mock.patch("boto3.session.Session")
    @mock.patch("airflow.providers.teradata.hooks.teradata.TeradataHook.run")
    def test_execute(self, mock_run, mock_session, mock_connection, mock_hook):
        access_key = "aws_access_key_id"
        access_secret = "aws_secret_access_key"
        mock_session.return_value = Session(access_key, access_secret)
        mock_session.return_value.access_key = access_key
        mock_session.return_value.secret_key = access_secret
        mock_session.return_value.token = None

        mock_connection.return_value = Connection()
        mock_hook.return_value = Connection()

        op = S3ToTeradataOperator(
            s3_source_key=S3_SOURCE_KEY,
            teradata_table=TERADATA_TABLE,
            aws_conn_id=AWS_CONN_ID,
            teradata_conn_id=TERADATA_CONN_ID,
            task_id=TASK_ID,
            dag=None,
        )
        op.execute(None)

        assert mock_run.call_count == 1



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import re
from abc import abstractmethod
from enum import Enum
from functools import cached_property
from typing import TYPE_CHECKING

from airflow.models import BaseOperator
from airflow.providers.teradata.hooks.teradata import TeradataHook
from airflow.providers.teradata.utils.constants import Constants

if TYPE_CHECKING:
    try:
        from airflow.sdk.definitions.context import Context
    except ImportError:
        # TODO: Remove once provider drops support for Airflow 2
        from airflow.utils.context import Context

from collections.abc import Sequence
from datetime import timedelta
from typing import TYPE_CHECKING, Any, cast

from airflow.providers.teradata.triggers.teradata_compute_cluster import TeradataComputeClusterSyncTrigger

if TYPE_CHECKING:
    try:
        from airflow.sdk.definitions.context import Context
    except ImportError:
        # TODO: Remove once provider drops support for Airflow 2
        from airflow.utils.context import Context

from airflow.exceptions import AirflowException


# Represents
# 1. Compute Cluster Setup - Provision and Decomission operations
# 2. Compute Cluster State - Resume and Suspend operations
class _Operation(Enum):
    SETUP = 1
    STATE = 2


# Handler to handle single result set of a SQL query
def _single_result_row_handler(cursor):
    records = cursor.fetchone()
    if isinstance(records, list):
        return records[0]
    if records is None:
        return records
    raise TypeError(f"Unexpected results: {cursor.fetchone()!r}")


# Providers given operation is setup or state operation
def _determine_operation_context(operation):
    if operation == Constants.CC_CREATE_OPR or operation == Constants.CC_DROP_OPR:
        return _Operation.SETUP
    return _Operation.STATE


class _TeradataComputeClusterOperator(BaseOperator):
    """
    Teradata Compute Cluster Base Operator to set up and status operations of compute cluster.

    :param compute_profile_name: Name of the Compute Profile to manage.
    :param compute_group_name: Name of compute group to which compute profile belongs.
    :param teradata_conn_id: The :ref:`Teradata connection id <howto/connection:teradata>`
        reference to a specific Teradata database.
    :param timeout: Time elapsed before the task times out and fails.
    """

    template_fields: Sequence[str] = (
        "compute_profile_name",
        "compute_group_name",
        "teradata_conn_id",
        "timeout",
    )

    ui_color = "#e07c24"

    def __init__(
        self,
        compute_profile_name: str,
        compute_group_name: str | None = None,
        teradata_conn_id: str = TeradataHook.default_conn_name,
        timeout: int = Constants.CC_OPR_TIME_OUT,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.compute_profile_name = compute_profile_name
        self.compute_group_name = compute_group_name
        self.teradata_conn_id = teradata_conn_id
        self.timeout = timeout

    @cached_property
    def hook(self) -> TeradataHook:
        return TeradataHook(teradata_conn_id=self.teradata_conn_id)

    @abstractmethod
    def execute(self, context: Context):
        pass

    def execute_complete(self, context: Context, event: dict[str, Any]) -> None:
        """
        Execute when the trigger fires - returns immediately.

        Relies on trigger to throw an exception, otherwise it assumes execution was successful.
        """
        self._compute_cluster_execute_complete(event)

    def _compute_cluster_execute(self):
        # Verifies the provided compute profile name.
        if (
            self.compute_profile_name is None
            or self.compute_profile_name == "None"
            or self.compute_profile_name == ""
        ):
            self.log.info("Invalid compute cluster profile name")
            raise AirflowException(Constants.CC_OPR_EMPTY_PROFILE_ERROR_MSG)
        # Verifies if the provided Teradata instance belongs to Vantage Cloud Lake.
        lake_support_find_sql = "SELECT count(1) from DBC.StorageV WHERE StorageName='TD_OFSSTORAGE'"
        lake_support_result = self.hook.run(lake_support_find_sql, handler=_single_result_row_handler)
        if lake_support_result is None:
            raise AirflowException(Constants.CC_GRP_LAKE_SUPPORT_ONLY_MSG)
        # Getting teradata db version. Considering teradata instance is Lake when db version is 20 or above
        db_version_get_sql = "SELECT  InfoData AS Version FROM DBC.DBCInfoV WHERE InfoKey = 'VERSION'"
        try:
            db_version_result = self.hook.run(db_version_get_sql, handler=_single_result_row_handler)
            if db_version_result is not None:
                db_version_result = str(db_version_result)
                db_version = db_version_result.split(".")[0]
                if db_version is not None and int(db_version) < 20:
                    raise AirflowException(Constants.CC_GRP_LAKE_SUPPORT_ONLY_MSG)
            else:
                raise AirflowException("Error occurred while getting teradata database version")
        except Exception as ex:
            self.log.error("Error occurred while getting teradata database version: %s ", str(ex))
            raise AirflowException("Error occurred while getting teradata database version")

    def _compute_cluster_execute_complete(self, event: dict[str, Any]) -> None:
        if event["status"] == "success":
            return event["message"]
        if event["status"] == "error":
            raise AirflowException(event["message"])

    def _handle_cc_status(self, operation_type, sql):
        create_sql_result = self._hook_run(sql, handler=_single_result_row_handler)
        self.log.info(
            "%s query ran successfully. Differing to trigger to check status in db. Result from sql: %s",
            operation_type,
            create_sql_result,
        )
        self.defer(
            timeout=timedelta(minutes=self.timeout),
            trigger=TeradataComputeClusterSyncTrigger(
                teradata_conn_id=cast("str", self.teradata_conn_id),
                compute_profile_name=self.compute_profile_name,
                compute_group_name=self.compute_group_name,
                operation_type=operation_type,
                poll_interval=Constants.CC_POLL_INTERVAL,
            ),
            method_name="execute_complete",
        )

        return create_sql_result

    def _hook_run(self, query, handler=None):
        try:
            if handler is not None:
                return self.hook.run(query, handler=handler)
            return self.hook.run(query)
        except Exception as ex:
            self.log.error(str(ex))
            raise

    def _get_initially_suspended(self, create_cp_query):
        initially_suspended = "FALSE"
        pattern = r"INITIALLY_SUSPENDED\s*\(\s*'(TRUE|FALSE)'\s*\)"
        # Search for the pattern in the input string
        match = re.search(pattern, create_cp_query, re.IGNORECASE)
        if match:
            # Get the value of INITIALLY_SUSPENDED
            initially_suspended = match.group(1).strip().upper()
        return initially_suspended


class TeradataComputeClusterProvisionOperator(_TeradataComputeClusterOperator):
    """

    Creates the new Computer Cluster with specified Compute Group Name and Compute Profile Name.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:TeradataComputeClusterProvisionOperator`

    :param compute_profile_name: Name of the Compute Profile to manage.
    :param compute_group_name: Name of compute group to which compute profile belongs.
    :param query_strategy: Query strategy to use. Refers to the approach or method used by the
        Teradata Optimizer to execute SQL queries efficiently within a Teradata computer cluster.
        Valid query_strategy value is either 'STANDARD' or 'ANALYTIC'. Default at database level is STANDARD.
    :param compute_map: ComputeMapName of the compute map. The compute_map in a compute cluster profile refers
        to the mapping of compute resources to a specific node or set of nodes within the cluster.
    :param compute_attribute: Optional attributes of compute profile. Example compute attribute
        MIN_COMPUTE_COUNT(1) MAX_COMPUTE_COUNT(5) INITIALLY_SUSPENDED('FALSE')
    :param teradata_conn_id: The :ref:`Teradata connection id <howto/connection:teradata>`
        reference to a specific Teradata database.
    :param timeout: Time elapsed before the task times out and fails.
    """

    template_fields: Sequence[str] = (
        "compute_profile_name",
        "compute_group_name",
        "query_strategy",
        "compute_map",
        "compute_attribute",
        "teradata_conn_id",
        "timeout",
    )

    ui_color = "#e07c24"

    def __init__(
        self,
        query_strategy: str | None = None,
        compute_map: str | None = None,
        compute_attribute: str | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.query_strategy = query_strategy
        self.compute_map = compute_map
        self.compute_attribute = compute_attribute

    def _build_ccp_setup_query(self):
        create_cp_query = "CREATE COMPUTE PROFILE " + self.compute_profile_name
        if self.compute_group_name:
            create_cp_query = create_cp_query + " IN " + self.compute_group_name
        if self.compute_map is not None:
            create_cp_query = create_cp_query + ", INSTANCE = " + self.compute_map
        if self.query_strategy is not None:
            create_cp_query = create_cp_query + ", INSTANCE TYPE = " + self.query_strategy
        if self.compute_attribute is not None:
            create_cp_query = create_cp_query + " USING " + self.compute_attribute
        return create_cp_query

    def execute(self, context: Context):
        """
        Initiate the execution of CREATE COMPUTE SQL statement.

        Initiate the execution of the SQL statement for provisioning the compute cluster within Teradata Vantage
        Lake, effectively creates the compute cluster.
        Airflow runs this method on the worker and defers using the trigger.
        """
        super().execute(context)
        return self._compute_cluster_execute()

    def _compute_cluster_execute(self):
        super()._compute_cluster_execute()
        if self.compute_group_name:
            cg_status_query = (
                "SELECT  count(1) FROM DBC.ComputeGroups WHERE UPPER(ComputeGroupName) = UPPER('"
                + self.compute_group_name
                + "')"
            )
            cg_status_result = self._hook_run(cg_status_query, _single_result_row_handler)
            if cg_status_result is not None:
                cg_status_result = str(cg_status_result)
            else:
                cg_status_result = 0
            if int(cg_status_result) == 0:
                create_cg_query = "CREATE COMPUTE GROUP " + self.compute_group_name
                if self.query_strategy is not None:
                    create_cg_query = (
                        create_cg_query + " USING QUERY_STRATEGY ('" + self.query_strategy + "')"
                    )
                self._hook_run(create_cg_query, _single_result_row_handler)
        cp_status_query = (
            "SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('"
            + self.compute_profile_name
            + "')"
        )
        if self.compute_group_name:
            cp_status_query += " AND UPPER(ComputeGroupName) = UPPER('" + self.compute_group_name + "')"
        cp_status_result = self._hook_run(cp_status_query, handler=_single_result_row_handler)
        if cp_status_result is not None:
            cp_status_result = str(cp_status_result)
            msg = f"Compute Profile {self.compute_profile_name} is already exists under Compute Group {self.compute_group_name}. Status is {cp_status_result}"
            self.log.info(msg)
            return cp_status_result
        create_cp_query = self._build_ccp_setup_query()
        operation = Constants.CC_CREATE_OPR
        initially_suspended = self._get_initially_suspended(create_cp_query)
        if initially_suspended == "TRUE":
            operation = Constants.CC_CREATE_SUSPEND_OPR
        return self._handle_cc_status(operation, create_cp_query)


class TeradataComputeClusterDecommissionOperator(_TeradataComputeClusterOperator):
    """
    Drops the compute cluster with specified Compute Group Name and Compute Profile Name.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:TeradataComputeClusterDecommissionOperator`

    :param compute_profile_name: Name of the Compute Profile to manage.
    :param compute_group_name: Name of compute group to which compute profile belongs.
    :param delete_compute_group: Indicates whether the compute group should be deleted.
        When set to True, it signals the system to remove the specified compute group.
        Conversely, when set to False, no action is taken on the compute group.
    :param teradata_conn_id: The :ref:`Teradata connection id <howto/connection:teradata>`
        reference to a specific Teradata database.
    :param timeout: Time elapsed before the task times out and fails.
    """

    template_fields: Sequence[str] = (
        "compute_profile_name",
        "compute_group_name",
        "delete_compute_group",
        "teradata_conn_id",
        "timeout",
    )

    ui_color = "#e07c24"

    def __init__(
        self,
        delete_compute_group: bool = False,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.delete_compute_group = delete_compute_group

    def execute(self, context: Context):
        """
        Initiate the execution of DROP COMPUTE SQL statement.

        Initiate the execution of the SQL statement for decommissioning the compute cluster within Teradata Vantage
        Lake, effectively drops the compute cluster.
        Airflow runs this method on the worker and defers using the trigger.
        """
        super().execute(context)
        return self._compute_cluster_execute()

    def _compute_cluster_execute(self):
        super()._compute_cluster_execute()
        cp_drop_query = "DROP COMPUTE PROFILE " + self.compute_profile_name
        if self.compute_group_name:
            cp_drop_query = cp_drop_query + " IN COMPUTE GROUP " + self.compute_group_name
        self._hook_run(cp_drop_query, handler=_single_result_row_handler)
        self.log.info(
            "Compute Profile %s IN Compute Group %s is successfully dropped",
            self.compute_profile_name,
            self.compute_group_name,
        )
        if self.delete_compute_group:
            cg_drop_query = "DROP COMPUTE GROUP " + self.compute_group_name
            self._hook_run(cg_drop_query, handler=_single_result_row_handler)
            self.log.info("Compute Group %s is successfully dropped", self.compute_group_name)


class TeradataComputeClusterResumeOperator(_TeradataComputeClusterOperator):
    """
    Teradata Compute Cluster Operator to Resume the specified Teradata Vantage Cloud Lake Compute Cluster.

    Resumes the Teradata Vantage Lake Computer Cluster by employing the RESUME SQL statement within the
    Teradata Vantage Lake Compute Cluster SQL Interface.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:TeradataComputeClusterResumeOperator`

    :param compute_profile_name: Name of the Compute Profile to manage.
    :param compute_group_name: Name of compute group to which compute profile belongs.
    :param teradata_conn_id: The :ref:`Teradata connection id <howto/connection:teradata>`
        reference to a specific Teradata database.
    :param timeout: Time elapsed before the task times out and fails. Time is in minutes.
    """

    template_fields: Sequence[str] = (
        "compute_profile_name",
        "compute_group_name",
        "teradata_conn_id",
        "timeout",
    )

    ui_color = "#e07c24"

    def __init__(
        self,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)

    def execute(self, context: Context):
        """
        Initiate the execution of RESUME COMPUTE SQL statement.

        Initiate the execution of the SQL statement for resuming the compute cluster within Teradata Vantage
        Lake, effectively resumes the compute cluster.
        Airflow runs this method on the worker and defers using the trigger.
        """
        super().execute(context)
        return self._compute_cluster_execute()

    def _compute_cluster_execute(self):
        super()._compute_cluster_execute()
        cc_status_query = (
            "SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('"
            + self.compute_profile_name
            + "')"
        )
        if self.compute_group_name:
            cc_status_query += " AND UPPER(ComputeGroupName) = UPPER('" + self.compute_group_name + "')"
        cc_status_result = self._hook_run(cc_status_query, handler=_single_result_row_handler)
        if cc_status_result is not None:
            cp_status_result = str(cc_status_result)
        # Generates an error message if the compute cluster does not exist for the specified
        # compute profile and compute group.
        else:
            self.log.info(Constants.CC_GRP_PRP_NON_EXISTS_MSG)
            raise AirflowException(Constants.CC_GRP_PRP_NON_EXISTS_MSG)
        if cp_status_result != Constants.CC_RESUME_DB_STATUS:
            cp_resume_query = f"RESUME COMPUTE FOR COMPUTE PROFILE {self.compute_profile_name}"
            if self.compute_group_name:
                cp_resume_query = f"{cp_resume_query} IN COMPUTE GROUP {self.compute_group_name}"
            return self._handle_cc_status(Constants.CC_RESUME_OPR, cp_resume_query)
        self.log.info(
            "Compute Cluster %s already %s", self.compute_profile_name, Constants.CC_RESUME_DB_STATUS
        )


class TeradataComputeClusterSuspendOperator(_TeradataComputeClusterOperator):
    """
    Teradata Compute Cluster Operator to suspend the specified Teradata Vantage Cloud Lake Compute Cluster.

    Suspends the Teradata Vantage Lake Computer Cluster by employing the SUSPEND SQL statement within the
    Teradata Vantage Lake Compute Cluster SQL Interface.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:TeradataComputeClusterSuspendOperator`

    :param compute_profile_name: Name of the Compute Profile to manage.
    :param compute_group_name: Name of compute group to which compute profile belongs.
    :param teradata_conn_id: The :ref:`Teradata connection id <howto/connection:teradata>`
        reference to a specific Teradata database.
    :param timeout: Time elapsed before the task times out and fails.
    """

    template_fields: Sequence[str] = (
        "compute_profile_name",
        "compute_group_name",
        "teradata_conn_id",
        "timeout",
    )

    ui_color = "#e07c24"

    def __init__(
        self,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)

    def execute(self, context: Context):
        """
        Initiate the execution of SUSPEND COMPUTE SQL statement.

        Initiate the execution of the SQL statement for suspending the compute cluster within Teradata Vantage
        Lake, effectively suspends the compute cluster.
        Airflow runs this method on the worker and defers using the trigger.
        """
        super().execute(context)
        return self._compute_cluster_execute()

    def _compute_cluster_execute(self):
        super()._compute_cluster_execute()
        sql = (
            "SEL ComputeProfileState FROM DBC.ComputeProfilesVX WHERE UPPER(ComputeProfileName) = UPPER('"
            + self.compute_profile_name
            + "')"
        )
        if self.compute_group_name:
            sql += " AND UPPER(ComputeGroupName) = UPPER('" + self.compute_group_name + "')"
        result = self._hook_run(sql, handler=_single_result_row_handler)
        if result is not None:
            result = str(result)
        # Generates an error message if the compute cluster does not exist for the specified
        # compute profile and compute group.
        else:
            self.log.info(Constants.CC_GRP_PRP_NON_EXISTS_MSG)
            raise AirflowException(Constants.CC_GRP_PRP_NON_EXISTS_MSG)
        if result != Constants.CC_SUSPEND_DB_STATUS:
            sql = f"SUSPEND COMPUTE FOR COMPUTE PROFILE {self.compute_profile_name}"
            if self.compute_group_name:
                sql = f"{sql} IN COMPUTE GROUP {self.compute_group_name}"
            return self._handle_cc_status(Constants.CC_SUSPEND_OPR, sql)
        self.log.info(
            "Compute Cluster %s already %s", self.compute_profile_name, Constants.CC_SUSPEND_DB_STATUS
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""Example Airflow DAG to show Stored Procedure creation and execution on teradata database using
TeradataStoredProcedureOperator.

This DAG assumes Airflow Connection with connection id `teradata_sp_call` already exists in locally. It
shows how to create and execute Stored Procedure as tasks in airflow dags using
TeradataStoredProcedureOperator."""

from __future__ import annotations

from datetime import datetime

import pytest

from airflow import DAG

try:
    from airflow.providers.teradata.operators.teradata import (
        TeradataOperator,
        TeradataStoredProcedureOperator,
    )
except ImportError:
    pytest.skip("Teradata provider apache-airflow-provider-teradata not available", allow_module_level=True)

# [START howto_teradata_operator_for_sp]
CONN_ID = "teradata_sp_call"
DAG_ID = "example_teradata_call_sp"

with DAG(
    dag_id=DAG_ID,
    max_active_runs=1,
    max_active_tasks=3,
    catchup=False,
    default_args={"teradata_conn_id": CONN_ID},
    schedule="@once",
    start_date=datetime(2023, 1, 1),
) as dag:
    # [START howto_teradata_stored_procedure_operator_with_in_inout_out]
    # [START howto_create_teradata_stored_procedure_operator_with_in_inout]
    create_sp_in_inout = TeradataOperator(
        task_id="create_sp_in_inout",
        sql=r"""REPLACE PROCEDURE TEST_PROCEDURE (
                    IN val_in INTEGER,
                    INOUT val_in_out INTEGER,
                    OUT val_out INTEGER,
                    OUT value_str_out varchar(100)
                )
                BEGIN
                    set val_out = val_in * 2;
                    set val_in_out = val_in_out * 4;
                    set value_str_out = 'string output';
                END;
            """,
    )
    # [END howto_create_teradata_stored_procedure_operator_with_in_inout]
    # [START howto_call_teradata_stored_procedure_operator_with_types]
    opr_sp_types = TeradataStoredProcedureOperator(
        task_id="opr_sp_types",
        procedure="TEST_PROCEDURE",
        parameters=[3, 1, int, str],
    )
    # [END howto_call_teradata_stored_procedure_operator_with_types]
    # [START howto_call_teradata_stored_procedure_operator_with_place_holder]
    opr_sp_place_holder = TeradataStoredProcedureOperator(
        task_id="opr_sp_place_holder",
        procedure="TEST_PROCEDURE",
        parameters=[3, 1, "?", "?"],
    )
    # [END howto_call_teradata_stored_procedure_operator_with_place_holder]
    # [START howto_call_teradata_stored_procedure_operator_with_dict_input]
    opr_sp_dict = TeradataStoredProcedureOperator(
        task_id="opr_sp_dict",
        procedure="TEST_PROCEDURE",
        parameters={"val_in": 3, "val_in_out": 1, "val_out": int, "str_out": str},
    )
    # [END howto_call_teradata_stored_procedure_operator_with_dict_input]
    # [END howto_teradata_stored_procedure_operator_with_in_inout_out]
    # [START howto_create_teradata_stored_procedure_operator_timestamp]
    create_sp_timestamp = TeradataOperator(
        task_id="create_sp_timestamp",
        sql=r"""REPLACE PROCEDURE GetTimestampOutParameter (OUT out_timestamp TIMESTAMP)
                    BEGIN
                        -- Assign current timestamp to the OUT parameter
                        SET out_timestamp = CURRENT_TIMESTAMP;
                    END;
                 """,
    )
    # [END howto_create_teradata_stored_procedure_operator_timestamp]
    # [START howto_call_teradata_stored_procedure_operator_timestamp]
    opr_sp_timestamp = TeradataStoredProcedureOperator(
        task_id="opr_sp_timestamp",
        procedure="GetTimestampOutParameter",
        parameters=["?"],
    )
    # [END howto_call_teradata_stored_procedure_operator_timestamp]
    # [START howto_teradata_stored_procedure_operator_with_in_out_dynamic_result]
    create_sp_param_dr = TeradataOperator(
        task_id="create_sp_param_dr",
        sql=r"""replace procedure examplestoredproc (in p1 integer, inout p2 integer, out p3 integer)
                dynamic result sets 2
                begin
                    declare cur1 cursor with return for select * from dbc.dbcinfo order by 1 ;
                    declare cur2 cursor with return for select infodata, infokey from dbc.dbcinfo order by 1 ;
                    open cur1 ;
                    open cur2 ;
                    set p2 = p1 + p2 ;
                    set p3 = p1 * p2 ;
                end ;
            """,
    )
    # [END howto_teradata_stored_procedure_operator_with_in_out_dynamic_result]
    # [START howto_call_teradata_stored_procedure_operator_with_in_out_dynamic_result]
    opr_sp_param_dr = TeradataStoredProcedureOperator(
        task_id="opr_sp_param_dr",
        procedure="examplestoredproc",
        parameters=[3, 2, int],
    )
    # [END howto_call_teradata_stored_procedure_operator_with_in_out_dynamic_result]
    # [START howto_teradata_stored_procedure_operator_drop]
    drop_sp = TeradataOperator(
        task_id="drop_sp",
        sql=r"drop procedure examplestoredproc;",
    )
    drop_sp_test = TeradataOperator(
        task_id="drop_sp_test",
        sql=r"drop procedure TEST_PROCEDURE;",
    )
    drop_sp_timestamp = TeradataOperator(
        task_id="drop_sp_timestamp",
        sql=r"drop procedure GetTimestampOutParameter;",
    )
    # [END howto_teradata_stored_procedure_operator_drop]
    (
        create_sp_in_inout
        >> opr_sp_types
        >> opr_sp_dict
        >> opr_sp_place_holder
        >> create_sp_param_dr
        >> opr_sp_param_dr
        >> drop_sp
        >> drop_sp_test
        >> create_sp_timestamp
        >> opr_sp_timestamp
        >> drop_sp_timestamp
    )

    # [END howto_teradata_operator_for_sp]

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG to show usage of teradata to teradata transfer operator.

The transfer operator connects to source teradata server, runs query to fetch data from source
and inserts that data into destination teradata database server. It assumes tables already exists.
The example DAG below assumes Airflow Connection with connection id `teradata_default` already exists.
It creates sample my_users table at source and destination, sets up sample data at source and then
runs transfer operator to copy data to corresponding table on destination server.
"""

from __future__ import annotations

import datetime
import os

import pytest

from airflow import DAG

try:
    from airflow.providers.teradata.operators.teradata import TeradataOperator
    from airflow.providers.teradata.transfers.teradata_to_teradata import TeradataToTeradataOperator
except ImportError:
    pytest.skip("Teradata provider apache-airflow-provider-teradata not available", allow_module_level=True)

# [START teradata_to_teradata_transfer_operator_howto_guide]

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "example_teradata_to_teradata_transfer_operator"
CONN_ID = "teradata_default"

with DAG(
    dag_id=DAG_ID,
    start_date=datetime.datetime(2020, 2, 2),
    schedule="@once",
    catchup=False,
    default_args={"teradata_conn_id": CONN_ID},
) as dag:
    # [START teradata_to_teradata_transfer_operator_howto_guide_create_src_table]
    create_src_table = TeradataOperator(
        task_id="create_src_table",
        sql="""
            CREATE TABLE my_users_src,
            FALLBACK (
                user_id decimal(10,0) NOT NULL GENERATED ALWAYS AS IDENTITY (
                    START WITH 1
                    INCREMENT BY 1
                    MINVALUE 1
                    MAXVALUE 2147483647
                    NO CYCLE),
                user_name VARCHAR(30),
                gender CHAR(1) DEFAULT 'M',
                birth_date DATE FORMAT 'YYYY-MM-DD' NOT NULL DEFAULT DATE '2023-01-01'
            ) PRIMARY INDEX (user_id);
        """,
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide_create_src_table]
    # [START teradata_to_teradata_transfer_operator_howto_guide_create_dest_table]
    create_dest_table = TeradataOperator(
        task_id="create_dest_table",
        sql="""
            CREATE TABLE my_users_dest,
            FALLBACK (
                user_id decimal(10,0) NOT NULL GENERATED ALWAYS AS IDENTITY (
                    START WITH 1
                    INCREMENT BY 1
                    MINVALUE 1
                    MAXVALUE 2147483647
                    NO CYCLE),
                user_name VARCHAR(30),
                gender CHAR(1) DEFAULT 'M',
                birth_date DATE FORMAT 'YYYY-MM-DD' NOT NULL DEFAULT DATE '2023-01-01'
            ) PRIMARY INDEX (user_id);
        """,
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide_create_dest_table]
    # [START teradata_to_teradata_transfer_operator_howto_guide_insert_data_src]
    insert_data_src = TeradataOperator(
        task_id="insert_data_src",
        sql="""
            INSERT INTO my_users_src(user_name) VALUES ('User1');
            INSERT INTO my_users_src(user_name) VALUES ('User2');
            INSERT INTO my_users_src(user_name) VALUES ('User3');
        """,
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide_insert_data_src]
    # [START teradata_to_teradata_transfer_operator_howto_guide_read_data_src]
    read_data_src = TeradataOperator(
        task_id="read_data_src",
        sql="SELECT TOP 10 * from my_users_src order by user_id desc;",
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide_read_data_src]
    # [START teradata_to_teradata_transfer_operator_howto_guide_transfer_data]
    transfer_data = TeradataToTeradataOperator(
        task_id="transfer_data",
        dest_teradata_conn_id="teradata_default",
        destination_table="my_users_dest",
        source_teradata_conn_id="teradata_default",
        sql="select * from my_users_src",
        sql_params={},
        rows_chunk=2,
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide_transfer_data]
    # [START teradata_to_teradata_transfer_operator_howto_guide_read_data_dest]
    read_data_dest = TeradataOperator(
        task_id="read_data_dest",
        sql="SELECT TOP 10 * from my_users_dest order by user_id desc;",
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide_read_data_dest]
    # [START teradata_to_teradata_transfer_operator_howto_guide_drop_src_table]
    drop_src_table = TeradataOperator(
        task_id="drop_src_table",
        sql=" DROP TABLE my_users_src;",
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide_drop_src_table]
    # [START teradata_to_teradata_transfer_operator_howto_guide_drop_dest_table]
    drop_dest_table = TeradataOperator(
        task_id="drop_dest_table",
        sql="DROP TABLE my_users_dest;",
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide_drop_dest_table]
    (
        create_src_table
        >> create_dest_table
        >> insert_data_src
        >> read_data_src
        >> transfer_data
        >> read_data_dest
        >> drop_src_table
        >> drop_dest_table
    )
    # [END teradata_to_teradata_transfer_operator_howto_guide]

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG to show usage of AzureBlobStorageToTeradataOperator

The transfer operator moves files in CSV, JSON, and PARQUET formats from Azure Blob storage
to Teradata tables. In the example Directed Acyclic Graph (DAG) below, it assumes Airflow
Connections with the IDs `teradata_default` and `wasb_default` already exist. The DAG creates
tables using data from the Azure Blob location, reports the number of rows inserted into
the table, and subsequently drops the table.
"""

from __future__ import annotations

import datetime
import os

import pytest

from airflow import DAG

try:
    from airflow.providers.teradata.operators.teradata import TeradataOperator
    from airflow.providers.teradata.transfers.azure_blob_to_teradata import AzureBlobStorageToTeradataOperator
except ImportError:
    pytest.skip("Teradata provider apache-airflow-provider-teradata not available", allow_module_level=True)

# [START azure_blob_to_teradata_transfer_operator_howto_guide]

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "example_azure_blob_to_teradata_transfer_operator"
CONN_ID = "teradata_default"

with DAG(
    dag_id=DAG_ID,
    start_date=datetime.datetime(2020, 2, 2),
    schedule="@once",
    catchup=False,
    default_args={"teradata_conn_id": CONN_ID},
) as dag:
    # [START azure_blob__to_teradata_transfer_operator_howto_guide_transfer_data_public_blob_to_teradata_csv]
    transfer_data_csv = AzureBlobStorageToTeradataOperator(
        task_id="transfer_data_blob_to_teradata_csv",
        blob_source_key="/az/akiaxox5jikeotfww4ul.blob.core.windows.net/td-usgs/CSVDATA/09380000/2018/06/",
        public_bucket=True,
        teradata_table="example_blob_teradata_csv",
        teradata_conn_id="teradata_default",
        azure_conn_id="wasb_default",
        trigger_rule="all_done",
    )
    # [END azure_blob__to_teradata_transfer_operator_howto_guide_transfer_data_public_blob_to_teradata_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    read_data_table_csv = TeradataOperator(
        task_id="read_data_table_csv",
        sql="SELECT count(1) from example_blob_teradata_csv;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    drop_table_csv = TeradataOperator(
        task_id="drop_table_csv",
        sql="DROP TABLE example_blob_teradata_csv;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_transfer_data_access_blob_to_teradata_csv]
    transfer_key_data_csv = AzureBlobStorageToTeradataOperator(
        task_id="transfer_key_data_blob_to_teradata_csv",
        blob_source_key="/az/airflowteradata.blob.core.windows.net/csvdata/",
        teradata_table="example_blob_teradata_csv",
        azure_conn_id="wasb_default",
        teradata_conn_id="teradata_default",
        trigger_rule="all_done",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_transfer_data_access_blob_to_teradata_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    read_key_data_table_csv = TeradataOperator(
        task_id="read_key_data_table_csv",
        conn_id=CONN_ID,
        sql="SELECT count(1) from example_blob_teradata_csv;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    drop_key_table_csv = TeradataOperator(
        task_id="drop_key_table_csv",
        conn_id=CONN_ID,
        sql="DROP TABLE example_blob_teradata_csv;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_create_authorization]
    create_azure_authorization = TeradataOperator(
        task_id="create_azure_authorization",
        conn_id=CONN_ID,
        sql="CREATE AUTHORIZATION azure_authorization USER '{{ var.value.get('AZURE_BLOB_ACCOUNTNAME') }}' PASSWORD '{{ var.value.get('AZURE_BLOB_ACCOUNT_SECRET_KEY') }}' ",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_create_authorization]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_transfer_data_authorization_blob_to_teradata_csv]
    transfer_auth_data_csv = AzureBlobStorageToTeradataOperator(
        task_id="transfer_auth_data_blob_to_teradata_csv",
        blob_source_key="/az/airflowteradata.blob.core.windows.net/csvdata/",
        teradata_table="example_blob_teradata_csv",
        teradata_authorization_name="azure_authorization",
        teradata_conn_id="teradata_default",
        trigger_rule="all_done",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_transfer_data_authorization_blob_to_teradata_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    read_auth_data_table_csv = TeradataOperator(
        task_id="read_auth_data_table_csv",
        conn_id=CONN_ID,
        sql="SELECT count(1) from example_blob_teradata_csv;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    drop_auth_table_csv = TeradataOperator(
        task_id="drop_auth_table_csv",
        conn_id=CONN_ID,
        sql="DROP TABLE example_blob_teradata_csv;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_csv]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_drop_authorization]
    drop_auth = TeradataOperator(
        task_id="drop_auth",
        conn_id=CONN_ID,
        sql="DROP AUTHORIZATION azure_authorization;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_drop_authorization]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_transfer_data_blob_to_teradata_json]
    transfer_data_json = AzureBlobStorageToTeradataOperator(
        task_id="transfer_data_blob_to_teradata_json",
        blob_source_key="/az/akiaxox5jikeotfww4ul.blob.core.windows.net/td-usgs/JSONDATA/09380000/2018/06/",
        teradata_table="example_blob_teradata_json",
        public_bucket=True,
        teradata_conn_id="teradata_default",
        azure_conn_id="wasb_default",
        trigger_rule="all_done",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_transfer_data_blob_to_teradata_json]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_json]
    read_data_table_json = TeradataOperator(
        task_id="read_data_table_json",
        sql="SELECT count(1) from example_blob_teradata_json;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_json]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_json]
    drop_table_json = TeradataOperator(
        task_id="drop_table_json",
        sql="DROP TABLE example_blob_teradata_json;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_json]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_transfer_data_blob_to_teradata_parquet]
    transfer_data_parquet = AzureBlobStorageToTeradataOperator(
        task_id="transfer_data_blob_to_teradata_parquet",
        blob_source_key="/az/akiaxox5jikeotfww4ul.blob.core.windows.net/td-usgs/PARQUETDATA/09394500/2018/06/",
        teradata_table="example_blob_teradata_parquet",
        public_bucket=True,
        teradata_conn_id="teradata_default",
        trigger_rule="all_done",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_transfer_data_blob_to_teradata_parquet]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_parquet]
    read_data_table_parquet = TeradataOperator(
        task_id="read_data_table_parquet",
        sql="SELECT count(1) from example_blob_teradata_parquet;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_read_data_table_parquet]
    # [START azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_parquet]
    drop_table_parquet = TeradataOperator(
        task_id="drop_table_parquet",
        sql="DROP TABLE example_blob_teradata_parquet;",
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide_drop_table_parquet]

    (
        transfer_data_csv
        >> transfer_data_json
        >> transfer_data_parquet
        >> read_data_table_csv
        >> read_data_table_json
        >> read_data_table_parquet
        >> drop_table_csv
        >> drop_table_json
        >> drop_table_parquet
        >> transfer_key_data_csv
        >> read_key_data_table_csv
        >> drop_key_table_csv
        >> create_azure_authorization
        >> transfer_auth_data_csv
        >> read_auth_data_table_csv
        >> drop_auth_table_csv
        >> drop_auth
    )
    # [END azure_blob_to_teradata_transfer_operator_howto_guide]

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)
